/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qfile.h>
#include <qmessagebox.h>	
#include <qregexp.h>
#include <stdlib.h> // realloc()
#include <ctype.h> // tolower()
#include <string.h> // strtok()
#include "plugins/zcomponent.h"
#include "cconfig.h"

int currentMin, currentMax;

CConfig::CConfig(QString name)
{
	module = 0;
	localName = name;
	readFile();
}

void CConfig::setModule(inComponent *mod)
{
	module = mod;
}

void CConfig::readFile()
{
	QFile file(localName.data());
	file.open( IO_ReadWrite );
	// le fichier est cree s'il n'existe pas

	int min, max;
	// indice min et max pour le tableau de verification

	currentMin = currentMax = -1;

	nb = nbCheck = 0;

	QString tmp;
	
	fileInfo = 0;
	checkInfo = 0;

	fileinfo fi;

	fi.doubleclick[0] = fi.title[0] = fi.recognition[0] = indiceRenvoie = 0;
	fi.doubleclickshift[0] = fi.doubleclickalt[0] = fi.doubleclickctrl[0] = 0;

	// maintenant on concatene toutes les lignes du fichiers dans la liste chainee
	while(file.atEnd() == FALSE)
	{
		file.readLine(buffer, 2048);
		buffer[strlen(buffer) - 1] = 0;


		total.append(buffer);
		// on stocke toutes les lignes dans la liste total
		// pour restituer le fichier dans sa totalite
		// lors de la sauvegarde

		// si c'est une ligne de commentaire, on s'en fout completement
		if (buffer[0] != '#')
		{
			//debug("Premier cars de la chaine : %d %d", buffer[0], buffer[1]);
			if (buffer[0] != 0)
			{
				tmp = buffer;
				
				if (tmp.left(6) == "title=") // titre
				{
					if (fi.title[0] > 0) 
					{
						// il y a avait quelque-chose
						// on est donc surement en train de traiter un nouveau type de fichier
						// on va donc sauver les infos avant de continuer

						nb++;
						fileInfo = (fileinfo *)realloc(fileInfo, nb * sizeof(fileinfo));
						parcours = fileInfo;
						strcpy((parcours + (nb - 1)) -> title, fi.title);
						strcpy((parcours + (nb - 1)) -> recognition, fi.recognition);
						strcpy((parcours + (nb - 1)) -> doubleclick, fi.doubleclick);
						strcpy((parcours + (nb - 1)) -> doubleclickshift, fi.doubleclickshift);
						strcpy((parcours + (nb - 1)) -> doubleclickalt, fi.doubleclickalt);
						strcpy((parcours + (nb - 1)) -> doubleclickctrl, fi.doubleclickctrl);
					
						// maintenant on analyse la verification (recognition)
						// et on la transforme un peu s'il le faut
						parseRecognition(fi.title, fi.recognition, &min, &max);
							
						// et on met a jour les indices de verification
						(parcours + (nb - 1)) -> mincheck = min;
						(parcours + (nb - 1)) -> maxcheck = max;

						//debug("min = %d, max = %d", min, max);
						//if (max >= 0)
						//{
							//for(int kk = min ; kk <= max ; kk++)
							//{
							//	debug("Nouvelle action enregistree:");
							//	debug("Pour le filetype : %d", nb - 1);
							//	debug("Type : %d", (checkInfo + kk) -> action);
							//	debug("Argument : %s", (checkInfo + kk) -> value);
							//}						
						//}
							
						fi.title[0] = fi.recognition[0] = fi.doubleclick[0] = 0;
						fi.doubleclickshift[0] = fi.doubleclickalt[0] = fi.doubleclickctrl[0] = 0;
															
					}

					strcpy(fi.title, tmp.right(tmp.length() - 6).data());
				}

				if (tmp.left(10) == "matchbyte=")
					if (fi.recognition[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.recognition, "\n");
						strcat(fi.recognition, tmp.data());
					}
					else // il n'y a rien
						strcpy(fi.recognition, tmp.data());

				if (tmp.left(15) == "matchcharacter=")
					if (fi.recognition[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.recognition, "\n");
						strcat(fi.recognition, tmp.data());
					}
					else // il n'y a rien
						strcpy(fi.recognition, tmp.data());

				if (tmp.left(21) == "matchcharacternocase=")
					if (fi.recognition[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.recognition, "\n");
						strcat(fi.recognition, tmp.data());
					}
					else // il n'y a rien
						strcpy(fi.recognition, tmp.data());
				if (tmp.left(10) == "matchname=")
					if (fi.recognition[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.recognition, "\n");
						strcat(fi.recognition, tmp.data());
					}
					else // il n'y a rien
						strcpy(fi.recognition, tmp.data());

				if (tmp.left(5) == "move=")
					if (fi.recognition[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.recognition, "\n");
						strcat(fi.recognition, tmp.data());
					}
					else // il n'y a rien
						strcpy(fi.recognition, tmp.data());

				if (tmp.left(7) == "moveto=")
					if (fi.recognition[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.recognition, "\n");
						strcat(fi.recognition, tmp.data());
					}
					else // il n'y a rien
						strcpy(fi.recognition, tmp.data());


				if (tmp.left(12) == "doubleclick=")
					if (fi.doubleclick[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.doubleclick, "\n");
						strcat(fi.doubleclick, tmp.right(tmp.length() - 12));
					}
					else // il n'y a rien
						strcpy(fi.doubleclick, tmp.right(tmp.length() - 12));

				if (tmp.left(17) == "doubleclickshift=")
					if (fi.doubleclickshift[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.doubleclickshift, "\n");
						strcat(fi.doubleclickshift, tmp.right(tmp.length() - 17));
					}
					else // il n'y a rien
						strcpy(fi.doubleclickshift, tmp.right(tmp.length() - 17));

				if (tmp.left(16) == "doubleclickctrl=")
					if (fi.doubleclickctrl[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.doubleclickctrl, "\n");
						strcat(fi.doubleclickctrl, tmp.right(tmp.length() - 16));
					}
					else // il n'y a rien
						strcpy(fi.doubleclickctrl, tmp.right(tmp.length() - 16));

				if (tmp.left(15) == "doubleclickalt=")
					if (fi.doubleclickalt[0] > 0) // il y a deja quelquechose
					{				
						strcat(fi.doubleclickalt, "\n");
						strcat(fi.doubleclickalt, tmp.right(tmp.length() - 15));
					}
					else // il n'y a rien
						strcpy(fi.doubleclickalt, tmp.right(tmp.length() - 15));
				
			}		

		}
	}

	if (fi.title[0] > 0) 
	{
		// il y a avait quelque-chose
		// on est donc surement en train de traiter un nouveau type de fichier
		// on va donc sauver les infos avant de continuer

		nb++;
		fileInfo = (fileinfo *)realloc(fileInfo, nb * sizeof(fileinfo));
		parcours = fileInfo;
		strcpy((parcours + (nb - 1)) -> title, fi.title);
		strcpy((parcours + (nb - 1)) -> recognition, fi.recognition);
		strcpy((parcours + (nb - 1)) -> doubleclick, fi.doubleclick);
		strcpy((parcours + (nb - 1)) -> doubleclickshift, fi.doubleclickshift);
		strcpy((parcours + (nb - 1)) -> doubleclickalt, fi.doubleclickalt);
		strcpy((parcours + (nb - 1)) -> doubleclickctrl, fi.doubleclickctrl);
					
		// maintenant on analyse la verification (recognition)
		// et on la transforme un peu s'il le faut
		parseRecognition(fi.title, fi.recognition, &min, &max);
							
		// et on met a jour les indices de verification
		(parcours + (nb - 1)) -> mincheck = min;
		(parcours + (nb - 1)) -> maxcheck = max;

		//debug("min = %d, max = %d", min, max);
		//if (max >= 0)
		//{
		//	for(int kk = min ; kk <= max ; kk++)
		//	{
		//		debug("Nouvelle action enregistree:");
		//		debug("Pour le filetype : %d", nb - 1);
		//		debug("Type : %d", (checkInfo + kk) -> action);
		//		debug("Argument : %s", (checkInfo + kk) -> value);
		//	}						
		//}
	}

	//debug("fin du scan");
	//debug("fileInfo : %d", nb);

	//debug("contenu de la liste");
	//for(int i = 0 ; i < nb ; i++)
	//{
	//	debug("title : %s", fileInfo[i].title);
	//	debug("extension : %s", fileInfo[i].extension);
	//	debug("header : %s", fileInfo[i].header);
	//	debug("link : %s", fileInfo[i].link);
	//	debug("action : %s", fileInfo[i].action);
	//}

	file.close();
}


int CConfig::addEntry()
{
	nb++;
	fileInfo = (fileinfo *)realloc(fileInfo, nb * sizeof(fileinfo));
	parcours = fileInfo;
	strcpy((parcours + (nb - 1)) -> title, "No Title");
	(parcours + (nb - 1)) -> recognition[0] = 0;
	(parcours + (nb - 1)) -> doubleclick[0] = 0;
	(parcours + (nb - 1)) -> doubleclickshift[0] = 0;
	(parcours + (nb - 1)) -> doubleclickalt[0] = 0;
	(parcours + (nb - 1)) -> doubleclickctrl[0] = 0;
	(parcours + (nb - 1)) -> mincheck = 0;
	(parcours + (nb - 1)) -> maxcheck = 0;

	return nb - 1;
}


void CConfig::removeEntry(int ind)
{
	//free(checkInfo);
	//checkInfo = 0;

	int j = 0;

	if (nb == 1)
	{
		free(fileInfo);
		fileInfo = 0;
		return;
	}
	else
	{
		// on cree un pointeur local
		fileinfo *tmp = (fileinfo *)malloc((nb - 1) * sizeof(fileinfo));
		//checkinfo *tmp2 = (checkinfo *)malloc((nb - 1) * sizeof(checkinfo));

		// et on retransfere tout sauf l'entree correspondante a l'indice
		for (int i = 0 ; i < nb ; i++)
		{
			if (i != ind) 
			{
			    strcpy((tmp + j) -> title, (fileInfo + i) -> title);
			    strcpy((tmp + j) -> recognition, (fileInfo + i) -> recognition);
			    strcpy((tmp + j) -> doubleclick, (fileInfo + i) -> doubleclick);
			    strcpy((tmp + j) -> doubleclickshift, (fileInfo + i) -> doubleclickshift);
			    strcpy((tmp + j) -> doubleclickalt, (fileInfo + i) -> doubleclickalt);
			    strcpy((tmp + j) -> doubleclickctrl, (fileInfo + i) -> doubleclickctrl);

			    (tmp + j) -> mincheck = (fileInfo + i) -> mincheck;
			    (tmp + j) -> maxcheck = (fileInfo + i) -> maxcheck;

			    j++;
			}
		}		

		nb--;		

		free(fileInfo);
		fileInfo = tmp;
	}
}


void CConfig::parseRecognition(const char *title, const char *recognition, int *min, int *max)
{
	currentMin = currentMax + 1;

	//unsigned int deb = 0;
	//unsigned int fin = 0;
	QString newValue;
	//bool good = FALSE;

	QString tmp = recognition;
	QString test;

	if (strlen(recognition) == 0)
	{
		*min = *max = 0;
		return;
	}

	//debug("Chaine de reconnaissance : %s", recognition);
	
	for (int i = 0 ; i <= tmp.length() ; i++)
	{
		//debug("%c -> %d", tmp.at(i), tmp.at(i));
		
		if (tmp.at(i) != 10 && tmp.at(i) != 0) // on concatene
			test += tmp.at(i);
		else
		{
			//debug("Chaine a analyser : %s", test.data());

			if (test.left(10) == "matchbyte=")
			{
				//debug("Chaine a analyser : %s", test.data());
				QString extracted;
				// valeur lue dans la chaine 

				unsigned int value;
				// valeur lue dans la chaine (convertie en int)

				bool toConvert = FALSE;
				// pour savoir si on doit convertir l'hexa en decimal

				unsigned int begin = 10;
				// a partir de quel caractere de la chaine on doit commencer a scruter	

				unsigned int leng = test.length();
				// longueur de la chaine
	
				if (test.left(12) == "matchbyte=0x")
				{
					begin += 2;
					toConvert = TRUE;
				}

			
				for (; begin < leng ;)
				{
					extracted = "";
					extracted += test.at(begin++);
					value = extracted.toInt();	

					if (toConvert == TRUE)
					{
						extracted += test.at(begin++);	
						if (extracted.at(0) >= '0' && extracted.at(0) <= '9')
							value = 16 * (extracted.at(0) - 48);
						else if (extracted.lower().at(0) >= 'a' && extracted.lower().at(0) <= 'f')
							value = 16 * (extracted.lower().at(0) - 87);
						else
						{
							QString error = "FileType : ";
							error += title;
							error += "\n";
							error += "Bad hexadecimal value : ";
							error += extracted.data();
							QMessageBox::critical(0, "Error", error.data());
							return;
						}

						if (extracted.at(1) >= '0' && extracted.at(1) <= '9')
							value += extracted.at(1) - 48;
						else if (extracted.lower().at(1) >= 'a' && extracted.lower().at(1) <= 'f')
							value += extracted.lower().at(1) - 87;
						else
						{
							QString error = "FileType : ";
							error += title;
							error += "\n";
							error += "Bad hexadecimal value : ";
							error += extracted.data();
							QMessageBox::critical(0, "Error", error.data());
							return;
						}

					} 
					else if (toConvert == FALSE)
					{
						while(test.at(begin) != ',' && begin < test.length())
						{
							//debug("%c -> %d", test.at(begin), test.at(begin));
	
							if (test.at(begin) >= '0' && test.at(begin) <= '9')
								value = value * 10 + test.at(begin) - 48;
							else if (test.at(begin) == ',' || test.at(begin) == 0)
								break;
							else
							{
								QString error = "FileType : ";
								error += title;
								error += "\n";
								error += "Bad decimal value : ";
								error += extracted.data();
								QMessageBox::critical(0, "Error", error.data());
								return;
							}

							extracted += test.at(begin++);
						}
					}

					QString tt;
					tt.sprintf("%d", value);

					newValue.append(tt.data());
					newValue.append(" ");
					//debug("Valeur trouvee : %d", value);
					//debug("Nouvelle chaine newValue : %s", newValue.data());
				}

				nbCheck++;
				checkInfo = (checkinfo *)realloc(checkInfo, nbCheck * sizeof(checkinfo));
				newValue.truncate(newValue.length() - 1);
		
				strcpy((checkInfo + (nbCheck - 1)) -> value, newValue.data());
				(checkInfo + (nbCheck - 1)) -> action = MATCHBYTE;
				currentMax++;
				
			//debug("Donc on va enregistrer : match='%s'", newValue.data());
			}
			else if(test.left(7) == "moveto=")
			{
				//debug("Chaine a analyser : %s", test.data());
				QString extracted;
				// valeur lue dans la chaine 

				unsigned int value;
				// valeur lue dans la chaine (convertie en int)

				bool toConvert = FALSE;
				// pour savoir si on doit convertir l'hexa en decimal

				unsigned int begin = 7;
				// a partir de quel caractere de la chaine on doit commencer a scruter	

				unsigned int leng = test.length();
				// longueur de la chaine

				if (test.left(9) == "moveto=0x")
				{
					begin = 9;
					toConvert = TRUE;
				}


				for (; begin < leng ;)
				{
//debug("ok");
					extracted = "";
					extracted += test.at(begin++);
					value = extracted.toInt();	
//debug("%d", value);

					if (toConvert == TRUE)
					{
						extracted += test.at(begin++);	
						if (extracted.at(0) >= '0' && extracted.at(0) <= '9')
							value = 16 * (extracted.at(0) - 48);
						else if (extracted.lower().at(0) >= 'a' && extracted.lower().at(0) <= 'f')
							value = 16 * (extracted.lower().at(0) - 87);
						else
						{
							QString error = "FileType : ";
							error += title;
							error += "\n";
							error += "Bad hexadecimal value : ";
							error += extracted.data();
							QMessageBox::critical(0, "Error", error.data());
							return;
						}

						if (extracted.at(1) >= '0' && extracted.at(1) <= '9')
							value += extracted.at(1) - 48;
						else if (extracted.lower().at(1) >= 'a' && extracted.lower().at(1) <= 'f')
							value += extracted.lower().at(1) - 87;
						else
						{
							QString error = "FileType : ";
							error += title;
							error += "\n";
							error += "Bad hexadecimal value : ";
							error += extracted.data();
							QMessageBox::critical(0, "Error", error.data());
							return;
						}

					} 
					else if (toConvert == FALSE)
					{
						while(begin < test.length())
						{
							//debug("%c -> %d", test.at(begin), test.at(begin));
	
							if (test.at(begin) >= '0' && test.at(begin) <= '9')
								value = value * 10 + test.at(begin) - 48;
							else if (test.at(begin) == 0)
								break;
							else
							{
								QString error = "FileType : ";
								error += title;
								error += "\n";
								error += "Bad decimal value : ";
								error += extracted.data();
								QMessageBox::critical(0, "Error", error.data());
								return;
							}

							extracted += test.at(begin++);
						}
					}

					QString tt;
					tt.sprintf("%d", value);

					newValue.append(tt.data());
					newValue.append(" ");
					//debug("Valeur trouvee : %d", value);
					//debug("Nouvelle chaine newValue : %s", newValue.data());
				}

				nbCheck++;
				checkInfo = (checkinfo *)realloc(checkInfo, nbCheck * sizeof(checkinfo));
		
				newValue.truncate(newValue.length() - 1);


				strcpy((checkInfo + (nbCheck - 1)) -> value, newValue.data());
				(checkInfo + (nbCheck - 1)) -> action = MOVETO;
				currentMax++;

			//debug("Donc on va enregistrer : match='%s'", newValue.data());

			}	
			else if (test.left(10) == "matchname=")
			{
				//debug("Chaine a analyser : %s", test.data());
				QString extracted;
				// valeur lue dans la chaine 

				unsigned int leng = test.length();
				// longueur de la chaine

				extracted = test.right(leng - 10);
	
				QRegExp exp(extracted.data());
				if (!exp.isValid())
				{
					QString error = "FileType : ";
					error += title;
					error += "\n";
					error += "Bad RegExpression : ";
					error += extracted.data();
					QMessageBox::critical(0, "Error", error.data());
					return;
				}


				nbCheck++;
				checkInfo = (checkinfo *)realloc(checkInfo, nbCheck * sizeof(checkinfo));
		
				strcpy((checkInfo + (nbCheck - 1)) -> value, extracted.data());
				(checkInfo + (nbCheck - 1)) -> action = MATCHNAME;
				currentMax++;
				
			//debug("Donc on va enregistrer : matchname='%s'", extracted.data());
			}
			else if (test.left(15) == "matchcharacter=")
			{
				//debug("Chaine a analyser : %s", test.data());
				QString extracted;
				// valeur lue dans la chaine 

				unsigned int leng = test.length();
				// longueur de la chaine

				extracted = test.right(leng - 15);
	
				nbCheck++;
				checkInfo = (checkinfo *)realloc(checkInfo, nbCheck * sizeof(checkinfo));
		
				strcpy((checkInfo + (nbCheck - 1)) -> value, extracted.data());
				(checkInfo + (nbCheck - 1)) -> action = MATCHCHAR;
				currentMax++;
				
			//debug("Donc on va enregistrer : matchcharacter='%s'", extracted.data());
			}
			else if (test.left(21) == "matchcharacternocase=")
			{
				//debug("Chaine a analyser : %s", test.data());
				QString extracted;
				// valeur lue dans la chaine 

				unsigned int leng = test.length();
				// longueur de la chaine

				extracted = test.right(leng - 21);
	
				nbCheck++;
				checkInfo = (checkinfo *)realloc(checkInfo, nbCheck * sizeof(checkinfo));
		
				strcpy((checkInfo + (nbCheck - 1)) -> value, extracted.data());
				(checkInfo + (nbCheck - 1)) -> action = MATCHCHARNOCASE;
				currentMax++;
				
			//debug("Donc on va enregistrer : matchcharacter='%s'", extracted.data());
			}
			

			//debug("Donc on va enregistrer : match=%s", newValue.data());
			test = "";	
			newValue = "";
		}
	}

	if (currentMin == currentMax && currentMax == -1)
		*min = *max = -1;
	else	
	{
		*min = currentMin;
		*max = currentMax;
	}
}

char *CConfig::getNextTitle()
{
    //debug("%d %d", indiceRenvoie, nb);
    if (indiceRenvoie >= nb)
    {
		indiceRenvoie = 0;
		return 0;
	}

	return (fileInfo + (indiceRenvoie++)) -> title;
}


fileinfo *CConfig::get(const char *n)
{
	fileinfo *retour = 0;
	QString test;

	//debug("Chaine de reconnaissance : %s (longueur = %d)", fileInfo -> recognition, strlen(fileInfo -> recognition));
		
	if (fileInfo == 0) return 0; // no fileTypes are registered
	
	if (strlen(fileInfo -> recognition) == 0) return 0;

	// on va scanner toute la liste des conditions de reconnaissanves de fichier
	cantOpen = FALSE;
	for (int i = 0 ; i < nb ; i++)
	{
		//debug("Passage filetype : %d", i);

		//cantOpen = FALSE;

		parcours = retour = (fileInfo + i);

		//debug("mincheck = %d, maxcheck = %d", parcours -> mincheck, parcours -> maxcheck);

		if (parcours -> mincheck == -1 || parcours -> maxcheck == -1)
		  return 0;

		//if (parcours -> mincheck == 0 && parcours -> maxcheck == 0)//{debug("0 et 0");
		//  return 0;//}

		if (alreadyOpened)
			module -> at(0);
		
		for (int j = parcours -> mincheck ; j <= parcours -> maxcheck ; j++)
		{

			if ((checkInfo + j) -> action == MATCHBYTE)
			{
				if (!checkMatchByte(n, QString((checkInfo + j) -> value)))
				// res > 0 -> c'est ok
				// res == 0 -> pas ok
					retour = 0;
			}

			else if ((checkInfo + j) -> action == MOVETO)
			{
				if(!checkMoveTo(n, QString((checkInfo + j) -> value)))
					retour = 0;
			}

			else if ((checkInfo + j) -> action == MATCHNAME)
			{
				if(checkMatchName(n, QString((checkInfo + j) -> value)) == -1)
					retour = 0;
			}

			else if ((checkInfo + j) -> action == MATCHCHAR)
			{
				if(!checkMatchCharacter(n, QString((checkInfo + j) -> value)))
					retour = 0;
			}

			else if ((checkInfo + j) -> action == MATCHCHARNOCASE)
			{
				if(!checkMatchCharacterNoCase(n, QString((checkInfo + j) -> value)))
					retour = 0;
			}

		}

		//debug("Retour = %d", retour);
		if (retour) break;

	}

	module -> close();
	alreadyOpened = FALSE;
	return retour;
}


int CConfig::checkMoveTo(const char *fileName, QString pattern)
{
	// on se deplace dans le fichier a partir du debut
	//debug("MoveTo : '%s' -> %d", pattern.data(), pattern.toInt());
	//debug("Move renvoie : %d", fileOpened.at(pattern.toInt()));

	return module -> at(pattern.toInt());

}


int CConfig::checkMatchByte(const char *fileName, QString pattern)
{
	//debug("checkMatch : '%s'", pattern.data());

	//unsigned int value = 0;
	// valeur qui correnspont a la chaine extraite et qui va etre comparee au contenu du fichier

	unsigned char read;
	// caractere lu dans le fichier

	//unsigned int leng = pattern.length();

	char *token;

	char *string = new char[pattern.length() + 1];
	strcpy(string, pattern.data());
	char *oldString = string;

	char delim[2];
	delim[0] = ' ';
	delim[1] = 0;

	//int found;
	// indice du caractere espace

	if (alreadyOpened == FALSE && cantOpen == FALSE)
	{
//debug("ouverture");
		//fileOpened.setName(fileName);
		if (module -> open(fileName, IO_ReadOnly) == FALSE)
		{
			debug("In checkMatch, unable to open the file %s in readonly mode", fileName);
			cantOpen = TRUE;
			return 0;
		}	
		module -> at(0);	
	
		alreadyOpened = TRUE;
	}

	token = strtok(oldString, delim);
	//debug("Premier appel a strtok : %s", token);

	read = module -> getch();
	//debug("Carac lu : '%c' -> %d", read, read);

	if (read != atoi(token))
	{
		//debug("premier renvoie faux");
		return FALSE;
	}

	while (token = strtok(0, delim))
	{
		read = module -> getch();
	//debug("Carac lu : '%c' -> %d", read, read);
		if (read != atoi(token))
	{
		//debug("deuxi renvoie faux");
			return FALSE;}
		//debug("Appels suivants a strtok : %s", token);
	}	


	return TRUE;
}


int CConfig::checkMatchName(const char *fileName, QString pattern)
{
    QRegExp exp(pattern.data());
    return exp.match(fileName);
}


int CConfig::checkMatchCharacter(const char *fileName, QString pattern)
{
    unsigned char read;
    // caractere lu dans le fichier

    if (alreadyOpened == FALSE && cantOpen == FALSE)
    {
		//fileOpened.setName(fileName);
		if (module -> open(fileName, IO_ReadOnly) == FALSE)
		{
			debug("In checkMatchCharacter, unable to open the file %s in readonly mode", fileName);
			cantOpen = TRUE;
			return 0;
		}	
		module -> at(0);	
	
		alreadyOpened = TRUE;
    }

    int len = pattern.length();

    for(int i = 0 ; i < len ; i++)
    {
		read = module -> getch();
		if (read != pattern.at(i)) return FALSE;
    }

    return TRUE;
}


int CConfig::checkMatchCharacterNoCase(const char *fileName, QString pattern)
{
    unsigned char read;
    // caractere lu dans le fichier

    if (alreadyOpened == FALSE && cantOpen == FALSE)
    {
		//fileOpened.setName(fileName);
		if (module -> open(fileName, IO_ReadOnly) == FALSE)
		{
			debug("In checkMatchCharacter, unable to open the file %s in readonly mode", fileName);
			cantOpen = TRUE;
			return 0;
		}	
		module -> at(0);	
	
		alreadyOpened = TRUE;
    }

    int len = pattern.length();

    for(int i = 0 ; i < len ; i++)
    {
		read = module -> getch();
		if (toupper(read) != toupper(pattern.at(i))) return FALSE;
    }

    return TRUE;
}


void CConfig::save()
{
	if (QFile::remove(localName.data()) == FALSE)
	{
		debug("Warning : could not delete the file %s, aborting", localName.data());
		return;
	}

	QFile file(localName.data());
	file.open(IO_WriteOnly);

	fileinfo *f;
	QString action;
	
	for(int i = 0 ;; i++)
	{
		f = getByIndex(i);
		if (f == 0) break;
		//debug("title=%s", f -> title);
		file.writeBlock("title=", 6);
		file.writeBlock(f -> title, strlen(f -> title));
		file.writeBlock("\n", 1);

		if (strlen(f -> doubleclick)) // il y a deja une action associee au doubleclick
			for (unsigned int i = 0 ; i <= strlen(f -> doubleclick) ; i++)
			{
				if (f -> doubleclick[i] != 10 && f -> doubleclick[i] != 0) 
					action += f -> doubleclick[i];
				else
				{
					//debug("doubleclick=%s", action.data());
					file.writeBlock("doubleclick=", 12);
					file.writeBlock(action.data(), action.length());
					file.writeBlock("\n", 1);
					action = "";
				}
			}

		action = "";

		if (strlen(f -> doubleclickshift)) // il y a deja une action associee au doubleclick + shift
			for (unsigned int i = 0 ; i <= strlen(f -> doubleclickshift) ; i++)
			{
				if (f -> doubleclickshift[i] != 10 && f -> doubleclickshift[i] != 0) 
					action += f -> doubleclickshift[i];
				else
				{
					//debug("doubleclickshift=%s", action.data());
					file.writeBlock("doubleclickshift=", 17);
					file.writeBlock(action.data(), action.length());
					file.writeBlock("\n", 1);
					action = "";
				}
			}

		action = "";

		if (strlen(f -> doubleclickctrl)) // il y a deja une action associee au doubleclick + ctrl
			for (unsigned int i = 0 ; i <= strlen(f -> doubleclickctrl) ; i++)
			{
				if (f -> doubleclickctrl[i] != 10 && f -> doubleclickctrl[i] != 0) 
					action += f -> doubleclickctrl[i];
				else
				{
					//debug("doubleclickctrl=%s", action.data());
					file.writeBlock("doubleclickctrl=", 16);
					file.writeBlock(action.data(), action.length());
					file.writeBlock("\n", 1);
					action = "";
				}
			}

		action = "";

		if (strlen(f -> doubleclickalt)) // il y a deja une action associee au doubleclick + alt
			for (unsigned int i = 0 ; i <= strlen(f -> doubleclickalt) ; i++)
			{
				if (f -> doubleclickalt[i] != 10 && f -> doubleclickalt[i] != 0) 
					action += f -> doubleclickalt[i];
				else
				{
					//debug("doubleclickalt=%s", action.data());
					file.writeBlock("doubleclickalt=", 15);
					file.writeBlock(action.data(), action.length());
					file.writeBlock("\n", 1);
					action = "";
				}
			}

		action = "";

		if (strlen(f -> recognition)) // il y a une sequence de reconnaissance
			for (unsigned int i = 0 ; i <= strlen(f -> recognition) ; i++)
			{
				if (f -> recognition[i] != 10 && f -> recognition[i] != 0) 
					action += f -> recognition[i];
				else
				{
					//debug("recognition=%s", action.data());
					

					//file.writeBlock("recognition=", 15);
					file.writeBlock(action.data(), action.length());
					file.writeBlock("\n", 1);
					action = "";
				}
			}
	}

	file.close();	

	// on relie le fichier
	readFile();
}
