/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef __CONFIG_H__
#define __CONFIG_H__

#include <qstring.h>
#include <qstrlist.h>
#include <qlist.h>
#include <qfile.h>
//#include "plugins/inModule.h"

struct fileinfo
{
	char title[40]; // description du type de fichiers
	char recognition[2048]; // criteres de reconnaisances du type (ils doivent etre separees par des \n)
	char doubleclick[2048]; // actions a entreprendre (elles doivent etre separees par des \n)
	char doubleclickshift[2048]; // actions a entreprendre (elles doivent etre separees par des \n)
	char doubleclickctrl[2048]; // actions a entreprendre (elles doivent etre separees par des \n)
	char doubleclickalt[2048]; // actions a entreprendre (elles doivent etre separees par des \n)

	unsigned int mincheck; // indice de la premiere verification a faire (ne pas toucher !!!!)
	unsigned int maxcheck; // indice de la derniere verification a faire (ne pas toucher !!!!)
};

class inComponent;

class CConfig
{
	public:
		CConfig(QString file);
		// ouvre le fichier selectionne

		void setModule(inComponent *mod);

		fileinfo *getByIndex(const unsigned int index) { return ((index >= nb) ? (0) : (fileInfo + index)); }
		// renvoie le type de fichier correspondant a l'indice

		fileinfo *get(const char *file);
		// essaye de detecter le type de fichier de 'file'
		// et renvoie les infos correspondantes
		// 0 si aucun type de fichier correspondant

		int addEntry();
		// rajoute un nouveau type de fichier intitule 'Sans titre'
		// et renvoie l'indice de ce type

		void removeEntry(int ind);
		// enleve l'entree qui correspond a l'indice ind

		char *getNextTitle();
		// renvoie le type de fichier suivant
		// ou 0 s'il n'y en a plus
		// appellee quand on doit construire une liste de type de fichiers

		void save();
		// sauvegarde la configuration sur le disque

		~CConfig() { save(); }
		// destructeur qui va remettre sur disque tous les changements effectues

	private:

		inComponent *module;

		void readFile();
		// va lire le fichier de configuration
		// par exemple au debut (lors de l'instanciation oubien apres avoir sauve)

		enum check_t { MATCHBYTE = 0, MATCHCHAR, MATCHCHARNOCASE, MATCHNAME, MOVETO, MOVE, NOACTION };

		struct checkinfo
		{
			check_t action; // type de verification a entreprendre ( exemple : MATCH)
			char value[2048]; // valeur de la verification ( exemple "32 44")
		};

		void parseRecognition(const char *, const char *, int *, int *);
		// va balayer la chaine de reconnaissance pour la reformater
		// et ainsi permettre aux fonctions Matchx de ne pas faire ce boulot en temps reel
		// ca devrait accelerer la reconnaissance des types de fichier

		int checkMatchByte(const char *, QString);
		int checkMatchName(const char *, QString);
		int checkMatchCharacter(const char *, QString);
		int checkMatchCharacterNoCase(const char *, QString);
		int checkMoveTo(const char *, QString);

		QStrList total;
		// on stocke dans cette liste toutes les lignes du fichier
		// pour pouvoir le restaurer intact a la terminaison du programme

		unsigned int indiceRenvoie;
		// indice qui correspond a la fonction getNextTitle

		bool changed;
		// pour savoir si on  doit reecrire les changements dans le fichier en quittant

		char buffer[2048];
		// tampon pour la lecture dans le fichier

		QString localName;

		bool alreadyOpened;
		// pour savoir si un fichier a deja ete ouvert par les fonction checkMatch, checkSeek etc
		// s'il a ete ouvert, on continue, sinon on l'ouvre avant de continuer

		//QFile fileOpened;

		unsigned int nb;
		// nombre de configs trouve

		unsigned int nbCheck;
		// nombre de verifications trouvees

		fileinfo *fileInfo, *parcours;

		checkinfo *checkInfo;

		bool cantOpen;
		// if a file can't be accessed, no need to reopen it many times !!
};

#endif
