/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef __CONFIGFT_H__
#define __CONFIGFT_H__

#include <qstring.h>
#include <qstrlist.h>
#include <qlist.h>
#include <qfile.h>
#include <qdir.h>


class CConfigOpt
{
	public:
		CConfigOpt(QString file, QDir current);
		// ouvre le fichier selectionne
		// obtient le repertoire courant
		// notamment pour mettre par defaut le repertoire des plugins
		// a la valeur du repertoire courant

		void clear() { total.clear(); }
		// clear all the options
		// you must fill it again after that !

		int addEntry(QString title, QString value);
		// rajoute une nouvelle entree dans les options
		// et renvoie l'indice de ce type
		// si une entree avec ce titre existe deja elle est remplacee par la nouvelle

		bool removeEntry(QString title);
		// enleve l'entree qui correspond a l'indice ind

		QString get(QString title, QString replacement);
		// recupere la valeur correspondante au titre indique

		void save();
		// sauvegarde la configuration sur le disque

		~CConfigOpt() {}
		// destructeur qui va remettre sur disque tous les changements effectues

	private:

		void readFile();
		// va lire le fichier de configuration
		// par exemple au debut (lors de l'instanciation oubien apres avoir sauve)

	

		QStrList total;
		// on stocke dans cette liste toutes les lignes du fichier
		// pour pouvoir le restaurer intact a la terminaison du programme


		bool changed;
		// pour savoir si on  doit reecrire les changements dans le fichier en quittant

		char buffer[2048];
		// tampon pour la lecture dans le fichier

		QString localName;
		QString localDir;

		QFile fileOpened;

		unsigned int nb;
		// nombre de configs trouve

		unsigned int nbCheck;
		// nombre de verifications trouvees

};

#endif
