/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qfiledialog.h>
#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#endif

#include "CActionAssociationDialog.h"
#include "CArgumentDialog.h"
#include "../internalCommands.h"

CActionAssociationDialog::CActionAssociationDialog(fileinfo *f, CConfigOpt *co, int n, QWidget *parent, const char *name)
	: actionAssociationDialog(parent, name), lf(f), ln(n), lco(co)
{
	mode = FILETYPE;

	del -> setEnabled(FALSE);
	insert -> setEnabled(FALSE);
	QString title = f -> title;
	QString action;

	if (n == 0)
	{
		if (strlen(f -> doubleclick)) // il y a deja une action associee au doubleclick
		for (unsigned int i = 0 ; i <= strlen(f -> doubleclick) ; i++)
		{
			if (f -> doubleclick[i] != 10 && f -> doubleclick[i] != 0) 
				action += f -> doubleclick[i];
			else
			{
				actions -> insertItem(action.data());
				doubleclick.append(action.data());
				action = "";
			}
		}

		title += " (Double click)";
	}
	else if (n == 1)
	{
		if (strlen(f -> doubleclickshift)) // il y a deja une action associee au doubleclick + shift
		for (unsigned int i = 0 ; i <= strlen(f -> doubleclickshift) ; i++)
		{
			if (f -> doubleclickshift[i] != 10 && f -> doubleclickshift[i] != 0) 
				action += f -> doubleclickshift[i];
			else
			{
				actions -> insertItem(action.data());
				doubleclickshift.append(action.data());
				action = "";
			}
		}

		title += " (Double click + Shift)";
	}
	else if (n == 2)
	{
		if (strlen(f -> doubleclickctrl)) // il y a deja une action associee au doubleclick + shift
		for (unsigned int i = 0 ; i <= strlen(f -> doubleclickctrl) ; i++)
		{
			if (f -> doubleclickctrl[i] != 10 && f -> doubleclickctrl[i] != 0) 
				action += f -> doubleclickctrl[i];
			else
			{
				actions -> insertItem(action.data());
				doubleclickctrl.append(action.data());
				action = "";
			}
		}

		title += " (Double click + Ctrl)";
	}
	else if (n == 3)
	{
		if (strlen(f -> doubleclickalt)) // il y a deja une action associee au doubleclick + shift
		for (unsigned int i = 0 ; i <= strlen(f -> doubleclickalt) ; i++)
		{
			if (f -> doubleclickalt[i] != 10 && f -> doubleclickalt[i] != 0) 
				action += f -> doubleclickalt[i];
			else
			{
				actions -> insertItem(action.data());
				doubleclickalt.append(action.data());
				action = "";
			}
		}

		title += " (Double click + Alt)";
	}
			
	setCaption(title.data());

}

CActionAssociationDialog::CActionAssociationDialog(ZToolbar *to, QString title, int index, QWidget *parent, const char *name)
	: actionAssociationDialog(parent, name), lf(0), ln(0), toolbar(to), currentCell(index)
{
	mode = TOOLBAR;

	del -> setEnabled(FALSE);
	insert -> setEnabled(FALSE);

	setCaption(title);

	QString action;
	char *act = toolbar -> getAction(currentCell);

	if (act > 0)
	{
		for (unsigned int i = 0 ; i <= strlen(act) ; i++)
		{
			if (act[i] != 10 && act[i] != 0) 
				action += act[i];
			else
			{
				actions -> insertItem(action.data());
				doubleclick.append(action.data());
				action = "";
			}
		}
	}
}

void CActionAssociationDialog::actionHighlighted(int n)
{
	if (ln == 0) // doubleclick
		edit -> setText(doubleclick.at(n));
	else if (ln == 1) // doubleclick + shift
		edit -> setText(doubleclickshift.at(n));
	else if (ln == 2) // doubleclick + ctrl
		edit -> setText(doubleclickctrl.at(n));
	else if (ln == 3) // doubleclick + alt
		edit -> setText(doubleclickalt.at(n));


	del -> setEnabled(TRUE);
	insert -> setEnabled(TRUE);
}


void CActionAssociationDialog::returnPressed()
{
	int number;

	if (ln == 0) // doubleclick
	{
		if (actions -> count())
		{
			number = actions -> currentItem();
			if (number == -1) 
			{
				number = doubleclick.count();
				actions -> insertItem(edit -> text());
			}
			else
			{
				actions -> changeItem(edit -> text(), number);
				doubleclick.remove(number);
			}

			doubleclick.insert(number, edit -> text());

		}
		else
		{
			actions -> insertItem(edit -> text());
			doubleclick.insert(0, edit -> text());
		}		

	}
	else if (ln == 1) // doubleclick + shift
	{
		if (actions -> count())
		{
			number = actions -> currentItem();
			if (number == -1) 
			{
				number = doubleclickshift.count();
				actions -> insertItem(edit -> text());
			}
			else
			{
				actions -> changeItem(edit -> text(), number);
				doubleclickshift.remove(number);
			}

			doubleclickshift.insert(number, edit -> text());

		}
		else
		{
			actions -> insertItem(edit -> text());
			doubleclickshift.insert(0, edit -> text());
		}		

	}
	else if (ln == 2) // doubleclick + ctrl
	{
		if (actions -> count())
		{
			number = actions -> currentItem();
			if (number == -1) 
			{
				number = doubleclickctrl.count();
				actions -> insertItem(edit -> text());
			}
			else
			{
				actions -> changeItem(edit -> text(), number);
				doubleclickctrl.remove(number);
			}

			doubleclickctrl.insert(number, edit -> text());

		}
		else
		{
			actions -> insertItem(edit -> text());
			doubleclickctrl.insert(0, edit -> text());
		}		

	}
	else if (ln == 3) // doubleclick + alt
	{
		if (actions -> count())
		{
			number = actions -> currentItem();
			if (number == -1) 
			{
				number = doubleclickalt.count();
				actions -> insertItem(edit -> text());
			}
			else
			{
				actions -> changeItem(edit -> text(), number);
				doubleclickalt.remove(number);
			}

			doubleclickalt.insert(number, edit -> text());

		}
		else
		{
			actions -> insertItem(edit -> text());
			doubleclickalt.insert(0, edit -> text());
		}		

	}

}


void CActionAssociationDialog::OnUse()
{
	if (mode == FILETYPE) 
	{
		if (ln == 0) // doubleclick
		{
			if (doubleclick.count() > 0)
				strcpy(lf -> doubleclick, doubleclick.at(0));
			else
				lf -> doubleclick[0] = 0;

			for (unsigned int i = 1 ; i < doubleclick.count() ; i++ )
			{
				if (strlen(doubleclick.at(i)))
				{
					strcat(lf -> doubleclick, "\n");
					strcat(lf -> doubleclick, doubleclick.at(i));
				}	
			}

			done(doubleclick.count());
		}
		else if (ln == 1) // doubleclick + shift
		{
			if (doubleclickshift.count() > 0)
				strcpy(lf -> doubleclickshift, doubleclickshift.at(0));
			else
				lf -> doubleclickshift[0] = 0;

			for (unsigned int i = 1 ; i < doubleclickshift.count() ; i++ )
			{
				if (strlen(doubleclickshift.at(i)))
				{
					strcat(lf -> doubleclickshift, "\n");
					strcat(lf -> doubleclickshift, doubleclickshift.at(i));
				}	
			}

			done(doubleclickshift.count());
		}
		else if (ln == 2) // doubleclick + ctrl
		{
			if (doubleclickctrl.count() > 0)
				strcpy(lf -> doubleclickctrl, doubleclickctrl.at(0));
			else
				lf -> doubleclickctrl[0] = 0;

			for (unsigned int i = 1 ; i < doubleclickctrl.count() ; i++ )
			{
				if (strlen(doubleclickctrl.at(i)))
				{
					strcat(lf -> doubleclickctrl, "\n");
					strcat(lf -> doubleclickctrl, doubleclickctrl.at(i));
				}	
			}

			done(doubleclickctrl.count());
		}
		else if (ln == 3) // doubleclick + alt
		{
			if (doubleclickalt.count() > 0)
				strcpy(lf -> doubleclickalt, doubleclickalt.at(0));
			else
				lf -> doubleclickalt[0] = 0;

			for (unsigned int i = 1 ; i < doubleclickalt.count() ; i++ )
			{
				if (strlen(doubleclickalt.at(i)))
				{
					strcat(lf -> doubleclickalt, "\n");
					strcat(lf -> doubleclickalt, doubleclickalt.at(i));
				}	
			}

			done(doubleclickalt.count());
		}
	}
	else if (mode == TOOLBAR)
	{
		QString ac;

		for(int i = 0 ; i < doubleclick.count() ; i++)
		{
			ac.append(doubleclick.at(i));
			if (i < doubleclick.count() - 1)
				ac.append("\n");
		}

		if (ac.length() > 0)
			toolbar -> setAction(currentCell, ac);
		else
			toolbar -> removeEntry(currentCell);
		done(0);
	}
}


void CActionAssociationDialog::OnCancel()
{
	done(-1);
}


void CActionAssociationDialog::OnAdd()
{
	
	int index = actions -> count();

	actions -> insertItem("", index);

	if (ln == 0) // doubleclick
		doubleclick.insert(index, "");
	else if (ln == 1) // doubleclick + shift
		doubleclickshift.insert(index, "");
	else if (ln == 2) // doubleclick + ctrl
		doubleclickctrl.insert(index, "");
	else if (ln == 3) // doubleclick + alt
		doubleclickalt.insert(index, "");
	

	actions -> setCurrentItem(index);
	edit -> setFocus();
	
}


void CActionAssociationDialog::OnDelete()
{
	int index = actions -> currentItem();
	actions -> removeItem(index);

	if (ln == 0) // doubleclick
		doubleclick.remove(index);
	else if (ln == 1) // doubleclick + shift
		doubleclickshift.remove(index);
	else if (ln == 2) // doubleclick + ctrl
		doubleclickctrl.remove(index);
	else if (ln == 3) // doubleclick + alt
		doubleclickalt.remove(index);
		

	if (actions -> count() == 0) 
	{
		del -> setEnabled(FALSE);
		insert -> setEnabled(FALSE);
	}
}


void CActionAssociationDialog::OnInsert()
{
	int index = actions -> currentItem();
	debug("current : %d", index);
	if (index == -1) index = 0;

	if (ln == 0) // doubleclick
		doubleclick.insert(index, "");
	else if (ln == 1) // doubleclick + shift
		doubleclickshift.insert(index, "");
	else if (ln == 2) // doubleclick + ctrl
		doubleclickctrl.insert(index, "");
	else if (ln == 3) // doubleclick + alt
		doubleclickalt.insert(index, "");
		
	actions -> insertItem("", index);

	actions -> setCurrentItem(index);

	edit -> setFocus();
}


void CActionAssociationDialog::OnArgument()
{
	QString arg1 = edit -> text(), arg;

	CArgumentDialog a(this);
	QString entry;

	for(int i = 0 ;; i++)
	{
	    if (arguments[i][0] == 0) break;
	    entry.sprintf("%s\t\t%s", arguments[i][0], arguments[i][1]);

	    a.addEntry(entry);
	}

	//a.addEntry("{f}\t\tFirst selected entry (with path)");
	//a.addEntry("{F}\t\tAll selected entries (with path)");
	//a.addEntry("{o}\t\tFirst selected entry (only filename)");
	//a.addEntry("{O}\t\tAll selected entries (only filename)");
	//a.addEntry("{d}\t\tDestination directory path");
	//a.addEntry("{s}\t\tSource directory path");
	//a.addEntry("{f!}\t\tAs f but not required");
	//a.addEntry("{F!}\t\tAs F but not required");
	//a.addEntry("{o!}\t\tAs o but not required");
	//a.addEntry("{O!}\t\tAs O but not required");
	//a.addEntry("{d!}\t\tAs d but not required");
	//a.addEntry("{s!}\t\tAs s but not required");
	//a.addEntry("{Rs}\t\tString requester");
	//a.addEntry("{RS}\t\tSecure string requester");
	//a.addEntry("{Rf}\t\tFile requester");
	//a.addEntry("{RF}\t\tFile requester (save)");
	//a.addEntry("{Rd}\t\tDirectory requester");
	//a.addEntry("{v}\t\tEnvironment variable");

	int res = a.exec();

	if (res < 0) return;

	arg = arguments[res][0];

	arg1 += arg;
	edit -> setText(arg1.data());
	//returnPressed();
}


void CActionAssociationDialog::OnWhat()
{
	int current = type -> currentItem();
	
	if (current == 0) // Internal Command
	{
	
		QString arg1 = edit -> text(), arg;

		CArgumentDialog a(this);
		QString entry;

		for(int i = 0 ;; i++)
		{
			if (commands[i][0] == 0) break;
			entry.sprintf("%s\t\t%s", commands[i][0], commands[i][1]);

			a.addEntry(entry);
		//a.addEntry("AnsiRead\t\tVisualiseur de fichier(ANSI)");
		//a.addEntry("CheckFit\t\tVerifie la taille disponible");
		//a.addEntry("Compare\t\tCompare deux fichiers");
		//a.addEntry("Confirm\t\tDemande confirmation");
		//a.addEntry("Copy\t\tCopie les fichiers");
		//a.addEntry("CopyAs\t\tCopie les fichiers en les renommant");
		//a.addEntry("CreateFileType\t\tCree un nouveau type de fichier");
		//a.addEntry("DateStamp\t\tChange la date des fichiers");
		//a.addEntry("Delete\t\tEfface les fichiers");
		//a.addEntry("DeviceList\t\tAffiche la liste des peripheriques");
		//a.addEntry("DiskCopy\t\tCopy un disque sur un autre");
		//a.addEntry("FindFile\t\tCherche les fichiers");
		//a.addEntry("FormatDisk\t\tFormater un disque");
		//a.addEntry("GetSizes\t\tCalcule la taille des fichiers");
		//a.addEntry("HexRead\t\tVisualiseur de fichier(HEXA)");
		//a.addEntry("Join\t\tJoin plusieurs fichiers en un seul");
		//a.addEntry("LoadButtons\t\tCharge une nouvelle barre de bouttons");
		//a.addEntry("LoadEnvironment\t\tCharge un evironnement different");
		//a.addEntry("LoadOptions\t\tCharge des options differentes");
		//a.addEntry("MakeDir\t\tCree un repertoire");
		//a.addEntry("MakeLink\t\tCree un lien vers un autre fichier");
		//a.addEntry("Move\t\tDeplace les fichiers");
		//a.addEntry("MoveAs\t\tDeplace des fichiers en les renommant");
		//a.addEntry("None\t\tDeselectionne tout");
		//a.addEntry("Parent\t\tSaute au repertoire parent");
		//a.addEntry("PrintDir\t\tImprime la liste des fichiers");
		//a.addEntry("Protect\t\tChange les attributs de protection");
		//a.addEntry("Rename\t\tRenomme les fichiers");
		//a.addEntry("Root\t\tSaute au repertoire racine");
		//a.addEntry("ScanDir\t\tRescanne le contenu du repertoire courant");
		//a.addEntry("Search\t\tCherche du texte dans des fichiers");
		//a.addEntry("Split\t\tPartage des fichiers en plusieurs");
		//a.addEntry("Toggle\t\tInverse la selection");
		}

		int res = a.exec();

		if (res < 0) return;

		arg = commands[res][0];

		arg1 += arg;
		edit -> setText(arg1);	
	}
	else if (current == 1) // executable
	{
		QString name = QFileDialog::getOpenFileName();

		char *final = new char[name.length() + 1];
		strcpy(final, name.data());
#ifdef WIN32
		GetShortPathName(final, final, name.length());
#endif

		if (strlen(final))//!name.isEmpty())
			edit -> setText(final/*name.data()*/);
	}
}
