/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#if !defined(WIN32)
#include <unistd.h> // fork()
#endif
#include <stdlib.h> // system()
#include "CClassDialog.h"
#include "CArgumentDialog.h"

CClassDialog::CClassDialog(fileinfo *f, CConfigOpt *co, QWidget *parent, const char *name)
	: classDialog(parent, name), lf(f), lco(co)
{
	what -> setEnabled(FALSE);
	argument -> setEnabled(FALSE);
	insert -> setEnabled(FALSE);
	remove -> setEnabled(FALSE);
	

	QString title = lf -> title;
	title += " recognition";
	setCaption(title.data());

	nameid -> setText(lf -> title);

	QString entry;
	QString added;

	//debug("Chaine de reconnaisance : %s", lf -> recognition);	

	for(int i = 0 ; i <= strlen(lf -> recognition) ; i++)
	{
		if(lf -> recognition[i] != 10 && lf -> recognition[i] != 0) // on concatene
			entry += lf -> recognition[i];
		else
		{
			//debug("Chaine extraite : %s", entry.data());
			if(entry.left(10) == "matchbyte=")
			{
				added = "Match Byte ";
				added += entry.right(entry.length() - 10);
				patterns -> insertItem(added.data());
				pattern.append(entry.right(entry.length() - 10));
			}

			if(entry.left(15) == "matchcharacter=")
			{
				added = "Match Character ";
				added += entry.right(entry.length() - 15);
				patterns -> insertItem(added.data());
				pattern.append(entry.right(entry.length() - 15));
			}

			if(entry.left(21) == "matchcharacternocase=")
			{
				added = "Match Character (No Case) ";
				added += entry.right(entry.length() - 21);
				patterns -> insertItem(added.data());
				pattern.append(entry.right(entry.length() - 21));
			}

			if(entry.left(7) == "moveto=")
			{
				added = "Move To ";
				added += entry.right(entry.length() - 7);
				patterns -> insertItem(added.data());
				pattern.append(entry.right(entry.length() - 7));
			}

			if(entry.left(5) == "move=")
			{
				added = "Move ";
				added += entry.right(entry.length() - 5);
				patterns -> insertItem(added.data());
				pattern.append(entry.right(entry.length() - 5));
			}

			if(entry.left(10) == "matchname=")
			{
				added = "Match Name ";
				added += entry.right(entry.length() - 10);
				patterns -> insertItem(added.data());
				pattern.append(entry.right(entry.length() - 10));
			}

			entry = "";
		}
	}	


	contenu();

}


void CClassDialog::OnAdd()
{
	debug("Avant le add");
	contenu();

	CArgumentDialog dg;
	dg.addEntry("Match Byte");
	dg.addEntry("Match Character");
	dg.addEntry("Match Character (No Case)");
	dg.addEntry("Match Name");
	dg.addEntry("Move");
	dg.addEntry("Move To");
	int res = dg.exec();

	if (res == 0) // Match Byte
		patterns -> insertItem("Match Byte");	
	else if (res == 1) // Match Character
		patterns -> insertItem("Match Character");
	else if (res == 2) // Match Character (No Case)
		patterns -> insertItem("Match Character (No Case)");	
	else if (res == 3) // Match Name
		patterns -> insertItem("Match Name");	
	else if (res == 4) // Move
		patterns -> insertItem("Move");	
	else if (res == 5) // Move To
		patterns -> insertItem("Move To");	

	pattern.append("");

	current = pattern.count() - 1;

	
	argument -> setEnabled(TRUE);
	argument -> setFocus();

	debug("Apres le add");
	contenu();
}	


void CClassDialog::OnUse()
{
	QString tt = nameid -> text();
	if (tt.length() > 0)
		strcpy(lf -> title, tt.data());

	QString arg;

	for (int i = 0 ; i < pattern.count() ; i ++)
	{
		tt = patterns -> text(i);
		//debug("%s", tt.data());

		if (tt.left(10) == "Match Name")
			arg += "matchname=";
		else if (tt.left(10) == "Match Byte")
			arg += "matchbyte=";

		if (tt.left(25) == "Match Character (No Case)")
			arg += "matchcharacternocase=";
		else if (tt.left(15) == "Match Character")
			arg += "matchcharacter=";
	    
		if (tt.left(7) == "Move To")
			arg += "moveto=";
		else if (tt.left(4) == "Move")
			arg += "move=";
		
		arg += pattern.at(i);

		if (i < pattern.count() - 1)
			arg += "\n";
	}

	debug("%s", arg.data());
	strcpy(lf -> recognition, arg.data());

	done(1);
}
	
		
void CClassDialog::OnCancel()
{
	done(-1);
}

void CClassDialog::OnSelect()
{
	QString file = QFileDialog::getOpenFileName();
	if (file.isEmpty())
		return;

	if (QPixmap::imageFormat(file) == 0) // format not recognized
	{
		QMessageBox::critical(0, "Error", "File format not recognized.");
		return;
	}

	QPixmap p(file.data());
	icon -> setPixmap(p);
}

void CClassDialog::OnRemoveIcon()
{}


void CClassDialog::OnView()
{
	QString file = QFileDialog::getOpenFileName();
	if (file.isEmpty())
		return;

	QString cmd = lco -> get("hexeditor", "") + " " + file;

#if !defined(WIN32)
	if(!fork())
	{
#endif
		system(cmd.data());
#if !defined(WIN32)
		exit(127);
	}
#endif
}


void CClassDialog::OnWhat()
{
	debug("Avant le add");
	contenu();

	CArgumentDialog dg;
	dg.addEntry("Match Byte");
	dg.addEntry("Match Character");
	dg.addEntry("Match Character (No Case)");
	dg.addEntry("Match Name");
	dg.addEntry("Move");
	dg.addEntry("Move To");
	int res = dg.exec();

	if (res == 0) // Match Byte
	{
		patterns -> insertItem("Match Byte");	
		pattern.append("Match Byte");
	}
	else if (res == 1) // Match Character
	{
		patterns -> insertItem("Match Character");	
		pattern.append("Match Character");
	}
	else if (res == 2) // Match Character (No Case)
	{
		patterns -> insertItem("Match Character (No Case)");
		pattern.append("Match Character (No Case)");
	}
	else if (res == 3) // Match Name
	{
		patterns -> insertItem("Match Name");	
		pattern.append("Match Name");
	}
	else if (res == 4) // Move
	{
		patterns -> insertItem("Move");	
		pattern.append("Move");
	}
	else if (res == 5) // Move To
	{
		patterns -> insertItem("Move To");	
		pattern.append("Move To");
	}
		
	argument -> setEnabled(TRUE);
	argument -> setFocus();

	debug("Apres le add");
	contenu();
}	


void CClassDialog::textChanged(const char *t)
{
	QString caption;

	if (strlen(t) > 0)
	{
		caption.sprintf("%s recognition", t);
		setCaption(caption.data());
		usebt -> setEnabled(TRUE);
	}
	else
	{
		caption = "No name defined";
		setCaption(caption.data());
		usebt -> setEnabled(FALSE);
	}
}		


void CClassDialog::patternsHighlighted(int n)
{
	current = n;

	QString tt = patterns -> text(n);

	debug("QListView : %s", tt.data());
	debug("QStrList : %s", pattern.at(n));

	argument -> setEnabled(TRUE);
	remove -> setEnabled(TRUE);

	if (tt.left(10) == "Match Name")
		show -> setText("Match Name");
	else if (tt.left(10) == "Match Byte")
		show -> setText("Match Byte");

	if (tt.left(25) == "Match Character (No Case)")
		show -> setText("Match Character (No Case)");
	else if (tt.left(15) == "Match Character")
		show -> setText("Match Character");
	
	if (tt.left(7) == "Move To")
		show -> setText("Move To");
	else if (tt.left(4) == "Move")
		show -> setText("Move");
		
	argument -> setText(pattern.at(n));
	argument -> setFocus();

}


void CClassDialog::returnPressed()
{
	debug("Avant le return");
	contenu();

	if (patterns -> count() == 0)
	{
		debug("(r)Avant le return");
		contenu();
		return;
	}

	QString arg;
			
	pattern.remove(current);
	pattern.insert(current, argument -> text());

	QString tt = patterns -> text(current);
	if (tt.left(10) == "Match Name")
		arg = "Match Name ";
	else if (tt.left(10) == "Match Byte")
		arg = "Match Byte ";

	if (tt.left(15) == "Match Character")
		arg = "Match Character ";

	if (tt.left(25) == "Match Character (No Case)")
		arg = "Match Character (No Case) ";

	if (tt.left(7) == "Move To")
		arg = "Move To ";
	else if (tt.left(4) == "Move")
		arg = "Move ";

	arg += argument -> text();
	patterns -> changeItem(arg.data(), current);	

	debug("Apres le return");
	contenu();
}


void CClassDialog::OnRemove()
{
	debug("Avant le remove");
	contenu();

	patterns -> removeItem(current);
	pattern.remove(current + 1);

	current--;

	if (current < 0)
	    current = 0;

	if (patterns -> count() == 0)
	{
		show -> setText("");
		argument -> setText("");
		argument -> setEnabled(FALSE);
		remove -> setEnabled(FALSE);
	}

	debug("Apres le remove");
	contenu();
}			
