#include "CActionAssociationDialog.h"
#include "CToolBarDialog.h"

CToolBarDialog::CToolBarDialog(CConfigToolBar *conf, ZToolbar *local, QWidget *parent, const char *name)
    : toolBarDialog(parent, name), config(conf), localToolBar(local)
{
    listBox -> addEntry(QString("Left mousebutton"));
    //listBox -> addEntry(QString("Right mousebutton"));

	listBox -> setEnabled(FALSE);
	
    add -> setEnabled(FALSE);
    Delete -> setEnabled(FALSE);
    remove -> setEnabled(FALSE);
	edit -> setEnabled(FALSE);

    toolbar -> setConfig(config);
    toolbar -> readConfig();

    connect(pixmapLBox, SIGNAL(clicked(int)), SLOT(pixmapListBoxClicked(int)));
    connect(pixmapLBox, SIGNAL(dblClicked(int)), SLOT(pixmapListBoxDblClicked(int)));
   
    connect(toolbar, SIGNAL(clicked(int)), SLOT(toolbarClicked(int)));

	connect(listBox, SIGNAL(clicked(int)), SLOT(myListBoxClicked(int)));
	connect(listBox, SIGNAL(dblClicked(int)), SLOT(myListBoxDblClicked(int)));

	currentAction = -1;
}

void CToolBarDialog::pixmapListBoxClicked(int index)
{
    currentPixmapLBox = index;
    add -> setEnabled(TRUE);
}

void CToolBarDialog::pixmapListBoxDblClicked(int index)
{
    OnAdd();
}

void CToolBarDialog::toolbarClicked(int index)
{
    //debug("%d", index);
    currentToolBar = index;

    if (toolbar -> getNumCols() > 0)
    	remove -> setEnabled(TRUE);

    toolbar -> blink(index);

    QString toPrint;
    if (index >= 0)
    {
        toPrint.sprintf("%d %s", index, pixmapInfo[toolbar -> getNumEntry(index)].name);
        label -> setText(toPrint.data());
		listBox -> setEnabled(TRUE);

		if (strlen(toolbar -> getAction(currentToolBar)) == 0)
			listBox -> setSelected(0, FALSE);
		else
			listBox -> setSelected(0, TRUE);

    }
    else
        label -> setText("No item selected");

}

void CToolBarDialog::OnAdd()
{
    toolbar -> addEntry(currentPixmapLBox, QString(""), currentPixmapLBox, QString(""));
}

void CToolBarDialog::OnRemove()
{
    toolbar -> removeEntry(currentToolBar);

    if (currentToolBar > toolbar -> getNumCols() - 1)
	currentToolBar = toolbar -> getNumCols() - 1;

    if (currentToolBar == -1)
	remove -> setEnabled(FALSE);

    QString toPrint;
    if (currentToolBar >= 0)
    {
        toPrint.sprintf("%d %s", currentToolBar, pixmapInfo[toolbar -> getNumEntry(currentToolBar)].name);
        label -> setText(toPrint.data());
		listBox -> setEnabled(TRUE);
    }
    else
	{
        label -> setText("No item selected");
		listBox -> setEnabled(FALSE);
	}

    toolbar -> blink(currentToolBar);
}

void CToolBarDialog::OnEdit()
{
	CActionAssociationDialog dlg(toolbar, QString("en construction"), currentToolBar, this);
	dlg.exec();

	if (toolbar -> getAction(currentToolBar) == 0)
		listBox -> setSelected(0, FALSE);
	else
		listBox -> setSelected(0, TRUE);
	
}

void CToolBarDialog::myListBoxClicked(int index)
{
	currentAction = index;
	edit -> setEnabled(TRUE);
}

void CToolBarDialog::myListBoxDblClicked(int index)
{
	OnEdit();
}

void CToolBarDialog::OnUse()
{
	config -> clear();
	localToolBar -> clear();

	for(int i = 0 ; i < toolbar -> getNumCols() ; i++)
	{
		config -> addEntry(toolbar -> getAction(i), toolbar -> getNumEntry(i), "", -1);
		localToolBar -> addEntry(toolbar -> getNumEntry(i), toolbar -> getAction(i), -1, "");
	}
		

	done(0);
}

void CToolBarDialog::OnSave()
{
	config -> clear();
	localToolBar -> clear();

	for(int i = 0 ; i < toolbar -> getNumCols() ; i++)
	{
		config -> addEntry(toolbar -> getAction(i), toolbar -> getNumEntry(i), "", -1);
		localToolBar -> addEntry(toolbar -> getNumEntry(i), toolbar -> getAction(i), -1, "");
	}

	config -> save();
	done(0);
}

void CToolBarDialog::OnCancel()
{
	done(-1);
}
