/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qapplication.h>
#include <qstring.h>
#include <qcursor.h>
#include <stdlib.h> // malloc(), system(), getenv()
#include <qmessagebox.h>
#if !defined(WIN32)
#include <unistd.h> // fork()
#endif

#include "fenetre.h"
#include "utils.h"
#include "dialogs/CFileTypesDialog.h"
#include "dialogs/CNewFileTypeDialog.h"
#include "dialogs/COptionsDialog.h"
#include "dialogs/CToolBarDialog.h"

#include "version.h"

#include "internalCommands.h"

//////////////////////////////////////////////////////////////////
////////////////////////// class ZFenetre ////////////////////////
//////////////////////////////////////////////////////////////////

ZFenetre::ZFenetre(CConfig *conf1, CConfigOpt *conf2, CConfigToolBar *conf3, QString path, ZManager *man, QString plug, QWidget *parent, const char *name)
	: QWidget(parent, name), manager(man)
{

	setCaption(zfm_version);
	killed = false;
	handleIn = handleOut = 0;
	inModule = 0;
	outModule = 0;

	edit = new ZLineEdit(this, "lineEdit");
  	edit -> setFocusPolicy(QWidget::ClickFocus);

  	label = new QLabel(this, "label");
  	label -> setFrameStyle(QLabel::Box | QLabel::Sunken);

	toolbar = new ZToolbar(this, "toolbar");
	toolbar -> setConfig(conf3);
	toolbar -> readConfig();
	//toolbar -> addEntry(0, 0);

	fichiers = new QPopupMenu(0);
	fichiers -> insertItem("New window");
	fichiers -> insertItem("About ...");
	fichiers -> insertItem("Quit");

	options = new QPopupMenu(0);
	options -> insertItem("FileTypes");
	options -> insertItem("Paths and external programs");
	options -> insertItem("Lister Toolbar");
	
	plugins = new QPopupMenu(0);

	boutton = new QPushButton("", this);
	boutton -> setFocusPolicy(QWidget::NoFocus);

	QStrList l = manager -> getOutPluginList();
	int count = l.count();
	debug("nombre d'elems : %d", count);
	QString temp;

	for(int i = 0 ; i < count ; i++)
	{
		temp = l.at(i);
		if (temp == "---")
			plugins -> insertSeparator();
		else
			plugins -> insertItem(temp);
	}

	if (plugins -> count() > 0)
	   	plugins -> insertSeparator();
	plugins -> insertItem("Rescan available plugins");

	inPopup = new QPopupMenu(0);
	// menu for scan plugin

	outPopup = new QPopupMenu(0);
	// menu for display plugin

  	menu = new QMenuBar(this);
 	menu -> insertItem("&Files" , fichiers, -1, 0);
	menu -> insertItem("&Scan options", inPopup, -1, 1);
	menu -> insertItem("&Display options", outPopup, -1, 2);
	menu -> insertItem("&Plugin selection", plugins, -1, 3);
	menu -> insertItem("&General options", options, -1, 4);

	configFileType = conf1;
	configOptions = conf2;
	configToolBar = conf3;

	//textEditor = configOptions -> get("texteditor", "vi");
	hexEditor = configOptions -> get("hexeditor", "ghex");
	pluginPath = configOptions -> get("pluginpath", ".");

	//debug("%s", textEditor.data());
	//debug("%s", hexEditor.data());
	//debug("%s", pluginPath.data());

  	//currentPlugin = 0;

	listeur = 0;

	//QStrList m = manager -> getInPluginList();
	//if (m.count() > 0) // we assume that the in_localdisk is here
	//{
	//	inPlugin = pluginPath;
	//	inPlugin += "/";
	//	inPlugin += m.at(0);
	//	changeInModule(inPlugin.data());
	//}
	
	inPlugin = pluginPath;
	inPlugin += "/in_localdisk";
	outPlugin = pluginPath;
	outPlugin += "/";
	outPlugin += plug;
	//QString in_localdisk = pluginPath;
	//in_local += "in_localdisk.so";

	currentDir = path;//configOptions -> get("currentDirectory", "/");

	tScan = new threadScan();
	//tDisplay = new threadDisplay();

	changeInModule(inPlugin.data());
	//changeInModule("./plugins/ftp.so");
	changeOutModule(outPlugin.data());

	QObject::connect(fichiers, SIGNAL(activated(int)), this, SLOT(fileMenuClicked(int)));
	QObject::connect(outPopup, SIGNAL(activated(int)), this, SLOT(outMenuClicked(int)));
	QObject::connect(inPopup, SIGNAL(activated(int)), this, SLOT(inMenuClicked(int)));
	QObject::connect(options, SIGNAL(activated(int)), this, SLOT(miscMenuClicked(int)));
	QObject::connect(plugins, SIGNAL(activated(int)), this, SLOT(pluginsMenuClicked(int)));

	QObject::connect(edit, SIGNAL(tabPressed()), this, SLOT(tabPressed()));
	QObject::connect(edit, SIGNAL(returnPressed()), this, SLOT(enterPressed()));

	QObject::connect(toolbar, SIGNAL(clicked(int)), this, SLOT(toolBarClicked(int)));

	QObject::connect(boutton, SIGNAL(clicked()), SLOT(killThread()));

	//connect(&timerScan, SIGNAL(timeout()), SLOT(timeOutScan()));
	//connect(&timerDisplay, SIGNAL(timeout()), SLOT(timeOutDisplay()));
}

void ZFenetre::killThread()
{
	debug("in killThread");
	killed = true;

	tScan -> Kill();
	boutton -> setText("Source");

	//else if (mode == DISPLAY1 || mode == DISPLAY2 || mode == DISPLAY3)
	//	timerDisplay.stop();
	
}


ZFenetre::~ZFenetre()
{
	//listeur = 0;
	//ZModule::unloadModule(handleIn);
	//ZModule::unloadModule(handleOut);
	//configOptions -> addEntry("currentDirectory", currentDir);
	//configOptions -> save();

//    manager -> windowClosed();
}

void ZFenetre::closeEvent(QCloseEvent *e)
{
    manager -> windowClosed(this);
    e -> accept();
}
	
// internal commands
void ZFenetre::selectAll()
{
    int count = list.count();

    for(int i = 1 ; i < count ; i++)
		list.at(i) -> setSelected(TRUE);

	boutton -> setText("Displaying");
    //--outModule -> display(list);
	boutton -> setText("Source");
}

void ZFenetre::none()
{
    int count = list.count();

    for(int i = 1 ; i < count ; i++)
		list.at(i) -> setSelected(FALSE);

	boutton -> setText("Displaying");
    //--outModule -> display(list);
	boutton -> setText("Source");
}

void ZFenetre::toggle()
{
    int count = list.count();

    for(int i = 1 ; i < count ; i++)
    {
		if ( list.at(i) -> isSelected() == TRUE )
			list.at(i) -> setSelected(FALSE);
		else
			list.at(i) -> setSelected(TRUE);
    }

	boutton -> setText("Displaying");
    //--outModule -> display(list);
	boutton -> setText("Source");
}


void ZFenetre::tabPressed()
{
	// completion algorithm, (C) 1998 myself :) 
	// quite good but i'm sure might be improved(speed ?)
	//debug("tab");

	ZList temp1, temp2;

	int display = -1;
	unsigned int i, j, found, min = 0, oldMin = 0, numElem = 0;
	QString s = edit -> text();
	

	// first go to the good directory
	// searching for the position of the last slash /

#if defined(WIN32)
	for (int t = 0 ; t < s.length() ; t++)
	{
		if(s.at(t) == '\\') s.at(t) = '/';
	}
#endif
	

	found = s.findRev('/', s.length() - 1);
	currentDir = s.left(found + 1);

	boutton -> setText("Scanning");
	Message *pmsg;
	Thread foo;
	tScan -> setPath(currentDir);
	tScan -> setMessenger(&foo);
	tScan -> Run();
  	//timerScan.start(0);
		
	while(true)
	{
		qApp -> processEvents(0);
		pmsg = (Message *)foo.GetMessage(false);
		if(pmsg) 
			break;
		//timeOutScan();
		if (killed == true) 
			return;
	}

	temp1 = tScan -> getList();
	//--temp1 = inModule -> scan(s.left(found + 1));
	boutton -> setText("Source");

	// then, we get in the list all entries that begin with the text in the lineEdit
	
	for(i = 0 ; i < temp1.count() ; i++)
	{
		if (!temp1.at(i) -> absFilePath().find(s.data()))
		{
 			temp2.append(temp1.at(i));
			numElem++;
		}
	}

	if (numElem == 1)
	{
		edit -> setText(temp2.at(0) -> absFilePath());
		if (temp2.at(0) -> isDir())
		{
			edit -> setText(temp2.at(0) -> absFilePath() + "/");
			currentDir = temp2.at(0) -> absFilePath();
		}

	}		
	else if (numElem > 1)
	{
		for(j = 0 ; j < numElem ; j++)
		{
			for(;;)
			{
				if (s.length() < min + found + 1) break;
				if (temp2.at(j) -> fileName().at(min) != s.at(min + found + 1)) break;
				//debug("%d, %c", min, s.at(min + found + 1));

				min++;
			}

			display = j;
			if (min == 0) break;

			oldMin = min;
			min = 0;
		}			

		// now, we try to complete by adding letters at the end
		// if there are severals files beginning with the same letter sequence
		if (display > -1)
		{		

			for(;;)
			{
				if(temp2.at(display) -> fileName().length() < oldMin + 1) break;
				char car = temp2.at(0) -> fileName().at(oldMin);
				//debug("carcou : %c", car);
				//debug("oldMin = %d", oldMin);
				for(i = 0 ; i < temp2.count() ; i++)
				{
					if (temp2.at(i) -> fileName().at(oldMin) != car) break;
				}
	
				if (temp2.current() -> fileName().at(oldMin) != car) break;
				oldMin++;
			}	
					
		}
		

		//debug("oldMin = %d", oldMin);
		if (display > -1)
		{
			//debug("'%s'", temp2.at(display) -> absFilePath().left(found + 1 + oldMin).data());
			edit -> setText( temp2.at(display) -> absFilePath().left(found + 1 + oldMin).data());
			if (temp2.at(display) -> isDir()) 
				currentDir = temp2.at(display) -> absFilePath();
		}
	}

	boutton -> setText("Displaying");	
	if (outModule)
		outModule -> display(temp2);
	else
	    outGLModule -> display(temp2);
	boutton -> setText("Source");

}


void ZFenetre::enterPressed()
{
	// to be changed !!!!

	QString s = edit -> text();
	if (inModule -> isGoodDir(s))
	{
		currentDir = edit -> text();
		mode = SCAN;
		boutton -> setText("Scanning");
		//inModule -> cd(currentDir);
		Message *pmsg;
		Thread foo;
		tScan -> setPath(currentDir);
		tScan -> setMessenger(&foo);
		tScan -> Run();
  		//timerScan.start(0);
		
		while(true)
		{
			qApp -> processEvents(0);
			pmsg = (Message *)foo.GetMessage(false);
			if(pmsg) 
				break;
			//timeOutScan();
			if (killed == true) 
				return;
		}

		list = tScan -> getList();
		boutton -> setText("Source");

		boutton -> setText("Displaying");
		if (outModule)
		//	tDisplay -> setComponent(outModule);
			outModule -> display(list);
		else
		//	tDisplay -> setGLComponent(outGLModule);
		    outGLModule -> display(list);

		//Message *pmsg1;
		//mode = DISPLAY;
		//Thread foo1;
		//tDisplay -> setList(list);
		//tDisplay -> setMessenger(&foo1);
		//tDisplay -> Run();
  		//timerDisplay.start(0);
		//list = inModule -> scan(currentDir);
		
		//while(true)
		//{
			//qApp -> processOneEvent();
			//if (killed == true)
			//	return;
			//pmsg1 =  (Message *)foo1.GetMessage(false);
			//if(pmsg1)
			//{
			//	delete pmsg1;
			//	break;
			//}
		
		//}
		
		//if(killed == true)
		//	return;

		boutton -> setText("Source");

	}
	else if (inModule -> isGoodFile(s))
		dblClickEvent(s, 0);
}				    

void ZFenetre::changeInModule(QString name)
{
	ZModule::unloadModule(handleIn);

#ifdef WIN32
	name += ".dll";
#else
	name += ".so";
#endif

	handleIn = ZModule::loadModule(name);

  	if(!handleIn)
	{
	  QMessageBox::critical(0, "Error", "Could not load plugin " + name + ". Please check your path.");
	  inModule = 0;
	  return;
	}

  	inModule = ZModule::getInModule(handleIn);

	configFileType -> setModule(inModule);

	QStrList menuList = inModule -> getMenu();
	// let's get this plugin's menu

	inPopup -> clear();
	inPopup -> setEnabled(FALSE);

	QString temp;

	for(unsigned int i = 0 ; i < menuList.count() ; i++)
	{
		temp = menuList.at(i);
		//debug("'%s'", temp.data());
		if (temp.mid(temp.length() - 2, 1) == "_")
		{
			if (temp.right(1) == "1")
			{
				inPopup -> insertItem(temp.left(temp.length() - 2), -1, i);
				inPopup -> setItemChecked(i, TRUE);
			}
			else if (temp.right(1) == "0")
			{
				inPopup -> insertItem(temp.left(temp.length() - 2), -1, i);
				inPopup -> setItemChecked(i, FALSE);
			}
			else
			{
				debug("Bad entry in the menu : %s", temp.data());
				exit(12);
			}
				
		}
		else if(temp == "---")
			inPopup -> insertSeparator();	
		else
			inPopup -> insertItem(temp, -1, i);
	}			
		
	if (inPopup -> count()) inPopup -> setEnabled(TRUE);
	
  	edit -> setText(currentDir);
  	//dir = QString(currentDir);

  	// now we can scan
	mode = SCAN;
	Message *pmsg;
	Thread foo;
	boutton -> setText("Scanning");
	//inModule -> cd(currentDir);
	tScan -> setComponent(inModule);
	tScan -> setPath(currentDir);
	tScan -> setMessenger(&foo);
	tScan -> Run();
  	//timerScan.start(0);
	//list = inModule -> scan(currentDir);
	
	while(true)
	{
		qApp -> processEvents(0);
		pmsg = (Message *)foo.GetMessage(false);
		if(pmsg) 
			break;
		//timeOutScan();

		if (killed == true)
			return;
	}
	
	//if(killed == true)
	//	return;

	//while(tScan -> Running() == TRUE) {}
	//list = inModule -> getList();
	boutton -> setText("Source");
	//listDevices = inModule -> getDevices();
	//list = inModule -> getDevices();
}

void ZFenetre::changeOutModule(QString name)
{
	int i;
	killed = false;
	//currentItem = 0;

	if (listeur) listeur -> hide();

#ifdef WIN32	
	name += ".dll";
#else
	name += ".so";
#endif
	//ZModule::unloadModule(handleOut);

	handleOut = ZModule::loadModule(name);

  	if (!handleOut)
	{
	  QMessageBox::critical(0, "Error", "Could not load plugin " + name + ". Please check your path.");
	  outModule = 0;	
	  listeur = 0;
	  return;
	}

  	outModule = ZModule::getOutModule(handleOut);

	if (outModule == 0) // humm ... it's maybe an opengl component, let's try it
	{
	    outGLModule = ZModule::getOutGLModule(handleOut);
	    listeur = outGLModule -> getNewWidget(configFileType, this, "listeur");
	}
	else	       
		listeur = outModule -> getNewWidget(configFileType, this, "listeur");


	QStrList menuList;

	if (outModule)
		menuList = outModule -> getMenu();
	else
	    menuList = outGLModule -> getMenu();

	outPopup -> clear();
	outPopup -> setEnabled(FALSE);

	QString temp;

	for(i = 0 ; i < menuList.count() ; i++)
	{
		temp = menuList.at(i);
		//debug("'%s'", temp.data());
		if (temp.mid(temp.length() - 2, 1) == "_")
		{
			if (temp.right(1) == "1")
			{
				outPopup -> insertItem(temp.left(temp.length() - 2), -1, i);
				outPopup -> setItemChecked(i, TRUE);
			}
			else if (temp.right(1) == "0")
			{
				outPopup -> insertItem(temp.left(temp.length() - 2), -1, i);
				outPopup -> setItemChecked(i, FALSE);
			}
			else
			{
				debug("Mauvais entree dans le menu : %s", temp.data());
				exit(12);
			}
				
		}
		else if(temp == "---")
			outPopup -> insertSeparator();	
		else
			outPopup -> insertItem(temp, -1, i);
	}			
		
	if (outPopup -> count()) outPopup -> setEnabled(TRUE);

	int hauteur = menu -> height();

  	listeur -> setMouseTracking(TRUE);

  	QObject::connect(listeur, SIGNAL(clicked(QString, int)), SLOT(clickEvent(QString, int)));
  	QObject::connect(listeur, SIGNAL(dblClicked(QString, int)), SLOT(dblClickEvent(QString, int)));
  	QObject::connect(listeur, SIGNAL(highlighted(QString)), SLOT(highlightEvent(QString)));

  	//int hauteur = menu -> height();

	listeur -> setGeometry(0, hauteur + 1 + 20 + 16 + 1, width(), height() - 59 - hauteur);	
	edit -> setGeometry(0, hauteur + 1, width(), 20);
	toolbar -> setGeometry(0, hauteur + 1 + 20 + 1, width(), 16);
	boutton -> setGeometry(width() - 59, hauteur + 1, 60, 20);
	label -> setGeometry(0, height() - 20, width(), 20);
	
	listeur -> show();

	boutton -> setText("Displaying");
	//if (outModule)
	//	outModule -> clear();
	//else
	//    outGLModule -> clear();
	
	list = tScan -> getList();
	//count = list.count();
	//if (outModule)
	//	tDisplay -> setComponent(outModule);
		//outModule -> display(list);
	//else
	//	tDisplay -> setGLComponent(outGLModule);
	    //outGLModule -> display(list);

	//Message *pmsg;
	mode = DISPLAY;
	//Thread foo;
	//tDisplay -> setList(list);
	//tDisplay -> setMessenger(&foo);
	//tDisplay -> Run();
  	//timerDisplay.start(0);
	//list = inModule -> scan(currentDir);
	
	//while(!stop)
	//{
	//	qApp -> processEvents(0);
	//	if (killed == true && mode == DISPLAY1)
	//	{
			if (outModule)
				outModule -> display(list);
			else
				outGLModule -> display(list);
	
	//		return;
	//	}
	//	timeOutDisplay();
	//}
	
	//if(killed == true)
	//	return;

	boutton -> setText("Source");
}

void ZFenetre::fileMenuClicked(int id)
{
    if (id == 0) // new window
		manager -> addWindow();
    else if (id == 2) // quit
		manager -> quit();
}

void ZFenetre::rescanPlugins()
{
    plugins -> clear();

	manager -> rescanPlugins();
    QStrList l = manager -> getOutPluginList();

    int count = l.count();

    if(count > 0)
    {
		for(int i = 0 ; i < count ; i++)
			plugins -> insertItem(l.at(i));
    }

	QStrList m = manager -> getInPluginList();
	if (m.count() > 0) // we assume that the in_localdisk is here
	{
		inPlugin = pluginPath;
		inPlugin += "/";
		inPlugin += m.at(0);
		changeInModule(inPlugin.data());
	}

    if (plugins -> count() > 0)
		plugins -> insertSeparator();

	plugins -> insertItem("Rescan available plugins");
}

void ZFenetre::outMenuClicked(int id)
{
	if (outModule)
		outModule -> menuClicked(id);
	else
	    outGLModule -> menuClicked(id);
	
	QStrList menuList;

	if (outModule)
		menuList = outModule -> getMenu();
	else
	    menuList = outGLModule -> getMenu();

	outPopup -> clear();
	outPopup -> setEnabled(FALSE);

	QString temp;

	for(unsigned int i = 0 ; i < menuList.count() ; i++)
	{
		temp = menuList.at(i);
		//debug("'%s'", temp.data());
		if (temp.mid(temp.length() - 2, 1) == "_")
		{
			if (temp.right(1) == "1")
			{
				outPopup -> insertItem(temp.left(temp.length() - 2), -1, i);
				outPopup -> setItemChecked(i, TRUE);
			}
			else if (temp.right(1) == "0")
			{
				outPopup -> insertItem(temp.left(temp.length() - 2), -1, i);
				outPopup -> setItemChecked(i, FALSE);
			}
			else
			{
				debug("Bad entry in the menu : %s", temp.data());
				exit(12);
			}
				
		}
		else if(temp == "---")
			outPopup -> insertSeparator();	
		else
			outPopup -> insertItem(temp, -1, i);
	}			
		
	if (outPopup -> count()) outPopup -> setEnabled(TRUE);
}


void ZFenetre::inMenuClicked(int id)
{
	inModule -> menuClicked(id);

	QStrList menuList = inModule -> getMenu();
	
	inPopup -> clear();
	inPopup -> setEnabled(FALSE);

	QString temp;

	for(unsigned int i = 0 ; i < menuList.count() ; i++)
	{
		temp = menuList.at(i);
		//debug("'%s'", temp.data());
		if (temp.mid(temp.length() - 2, 1) == "_")
		{
			if (temp.right(1) == "1")
			{
				inPopup -> insertItem(temp.left(temp.length() - 2), -1, i);
				inPopup -> setItemChecked(i, TRUE);
			}
			else if (temp.right(1) == "0")
			{
				inPopup -> insertItem(temp.left(temp.length() - 2), -1, i);
				inPopup -> setItemChecked(i, FALSE);
			}
			else
			{
				debug("Bad entry in the menu : %s", temp.data());
				exit(12);
			}
				
		}
		else if(temp == "---")
			inPopup -> insertSeparator();	
		else
			inPopup -> insertItem(temp, -1, i);
	}			
		
	if (inPopup -> count()) inPopup -> setEnabled(TRUE);

  	setCursor(QCursor(waitCursor));

	boutton -> setText("Scanning");
  	//--list = inModule -> scan(currentDir);
	boutton -> setText("Source");

  	edit -> setText(currentDir);

	boutton -> setText("Displaying");
	//--if (outModule)
	//--	outModule -> display( list );
	//--else
	//--    outGLModule -> display( list );
	boutton -> setText("Source");

  	setCursor(QCursor(arrowCursor));
}

void ZFenetre::miscMenuClicked(int id)
{
	char *t;	

	if (id == 0) // fileTypes
	{
		CFileTypesDialog cf(configFileType, configOptions, this);
		for(;;)
		{
			t = configFileType -> getNextTitle();
			//debug("%s", t);
			if (t)
				cf.addEntry(QString(t));
			else
				break;
		}
	
		cf.exec();
	}

	else if (id == 1) // paths
	{
	  COptionsDialog cf(configOptions, this);
	  if (cf.exec()) // let's re-read options
	  {
	    //textEditor = configOptions -> get("texteditor", "vi");
	    hexEditor = configOptions -> get("hexeditor", "ghex");
	    pluginPath = configOptions -> get("pluginpath", ".");
	  }  
	}

	else if (id == 2) // toolBar
	{
	    CToolBarDialog dlg(configToolBar, toolbar, this);
	    int res = dlg.exec();
		if (res == 0)
		{
			toolbar -> clear();
			toolbar -> readConfig();
		}
		//	toolbar -> repaint();
	}  
}

void ZFenetre::pluginsMenuClicked(int id)
{
	debug("pluginsmenu : %d", id);

	if (id == plugins -> count() - 1) // rescan plugins
	    rescanPlugins();
	else
	{
	    outPlugin = pluginPath.copy();
	    outPlugin += "/";
		outPlugin += plugins -> text(id);
	    changeOutModule(outPlugin.data());
	}
}

void ZFenetre::toolBarClicked(int index)
{
	if (index > -1)
		debug("%s", toolbar -> getAction(index));
}

void ZFenetre::resizeEvent(QResizeEvent *e)
{
	int hauteur = menu -> height();

	if (listeur)
	{ 
		listeur -> setGeometry(0, hauteur + 1 + 20 + 16 + 1, width(), height() - 59 - hauteur);	

		edit -> setGeometry(0, hauteur + 1, width() - 60, 20);

		boutton -> setGeometry(width() - 59, hauteur + 1, 60, 20);
	  
		toolbar -> setGeometry(0, hauteur + 1 + 20 + 1, width(), 16);

		label -> setGeometry(0, height() - 20, width(), 20);
	}
	else
	{  
	  edit -> setGeometry(0, 0, 0, 0);

	  boutton -> setGeometry(0, 0, 0, 0);
	  
	  toolbar -> setGeometry(0, 0, 0, 0);

	  label -> setGeometry(0, 0, 0, 0);
	}  

}


void ZFenetre::keyPressEvent(QKeyEvent *e)
{
}	


QString ZFenetre::getCurrentDir()
{
    return currentDir;
}


void ZFenetre::clickEvent(QString name, int qualifier)
{
  	//dblClickEvent(name);
}


void ZFenetre::dblClickEvent(QString name, int qualifier)
{
	//dir = "";
	killed = false;
	int nbFound = 0;
	QString commande, arg, temp;
	//debug("%s", name.data());

  	//if (name == "..")
    //	currentDir = inModule -> cdUp();
  	/*else*/ if (inModule -> isGoodDir(name)) // is it a valid directory ?
	{
		if (name == "..")
		{
			boutton -> setText("Scanning");
    		currentDir = inModule -> cdUp();
			boutton -> setText("Source");
		}
		else	
    		currentDir = name;

		if (currentDir != QString(""))
		{
			//debug("New directory to scan : %s", currentDir.data());
  			setCursor(QCursor(waitCursor));

			boutton -> setText("Scanning");
			mode = SCAN;
			//inModule -> cd(currentDir);
			Message *pmsg;
			Thread foo;
			tScan -> setPath(currentDir);
			tScan -> setMessenger(&foo);
			tScan -> Run();
  			//timerScan.start(0);
			
			while(true)
			{
				qApp -> processEvents(0);
				pmsg = (Message *)foo.GetMessage(false);
				if(pmsg) 
					break;
				//timeOutScan();
				if (killed == true) 
					return;
			}

			list = tScan -> getList();
			boutton -> setText("Source");

  			edit -> setText(currentDir);

			boutton -> setText("Displaying");
			if (outModule)
				//tDisplay -> setComponent(outModule);
				outModule -> display(list);
			else
				//tDisplay -> setGLComponent(outGLModule);
				outGLModule -> display(list);

			//Message *pmsg1;
			mode = DISPLAY;
			
			//Thread foo1;
			//tDisplay -> setList(list);
			//tDisplay -> setMessenger(&foo1);
			//tDisplay -> Run();
  			//list = inModule -> scan(currentDir);
			//timerDisplay.start(0);
			//mode = DISPLAY1;
			//currentItem = 0;
			//count = list.count();
			//stop = false;
			//while(!stop)
			//{
			//	qApp -> processEvents(0);
			//	if (killed == true && mode == DISPLAY1)
			//	{
			//		if (outModule)
			//			outModule -> displayCanceled();
			//		else
			//			outGLModule -> displayCanceled();

			//		return;
			//	}
			//	timeOutDisplay();
			//}
			
			//if(killed == true)
			//	return;

			boutton -> setText("Source");

  			setCursor(QCursor(arrowCursor));
		}
	}
	else // it's a file
	{
		fileinfo *res = configFileType -> get(name);

		if (res > 0) // we found a filetype handled by zfm !!!
		{
			debug("You selected : %s", res -> title);
			//debug("Command to execute : %s", resfinal -> doubleclick);

			if (!qualifier) // no qualifier
				arg = res -> doubleclick;
			else if ((qualifier & ShiftButton) != 0) // with shift
				arg = res -> doubleclickshift;
			if ((qualifier & ControlButton) != 0) // with control
				arg = res -> doubleclickctrl;
			if ((qualifier & AltButton) != 0) // with alt
				arg = res -> doubleclickalt;
			

			int found;
			for(;;)
			{

#ifdef WIN32
				char *final = new char[name.length() + 1];
				strcpy(final, name.data());
				GetShortPathName(final, final, name.length());

				QString p(final);
#else

			  QString p(name);
			  p.prepend("\042");
			  p.append("\042");
#endif
/*#ifdef WIN32
			  for(int i = 0 ; i < p.length() ; i++)
			  {
				  if(p.at(i) == '/')
					  p.replace(i, 1, "\\");
			  }
#endif*/

			  found = arg.find("{f}");
			  if (found >= 0)
			    arg.replace(found, 3, p.data());
			  else
			    break;
			}

			//debug("I'm going to execute : %s", arg.data());

			commande += arg;
			found = -1;
			int found2;
			    
			for(;;)
		    {
			    found2 = commande.find('\n', found + 1);
			    if (found2 == -1) break;
			    temp = commande.mid(found + 1, found2 - found - 1);
			    nbFound++;
				temp.prepend("\042");
				temp.append("\042");
			    debug("%s", temp.data());
			    if (isInternalCommand(temp) >= 0) 
					executeCommand(temp, name);
			    else
			    {
#if !defined(WIN32)
					if(!fork())
					{
#endif
						system(temp.data());
#if !defined(WIN32)						
						exit(127);
					}
#endif
				}
				    
			    found = found2;
			}
		
			temp = commande.right(commande.length() - found - 1);
			//temp.prepend("\042");
			//temp.append("\042");
			//if (nbFound == 0)
			//{
			    debug("%s", commande.data());
			    if (isInternalCommand(temp) >= 0)
				executeCommand(temp, name);
			    else
			    {
#if !defined(WIN32)
					if(!fork())
					{
#endif
						system(temp.data());
#if !defined(WIN32)
						exit(127);
					}
#endif
				}
			//}

			    
		}
		else
		{
			// first we test if it's a text file
			// let's assume that a text file can be found
			// if the first 30 bytes are letters

			bool isText = TRUE;

			inModule -> open(name, IO_ReadOnly);
			//QFile file(name);
			//file.open( IO_ReadOnly );
			char read;	

			for(int i = 0 ; i < 30 ; i++)
			{
			    if (inModule -> atEnd()) break;
			    read = inModule -> getch();
			    if (!isprint(read) && read != '\n' && read != '\t' && read != '\r')
			    {
					isText = FALSE;
					break;
			    }
			}
			inModule -> close();

			if (isText)
			{
				    internalCommands *p = new internalCommands;
				    p -> textViewer(name);
				    p -> show();
			}
			else
			{	

				int res = QMessageBox::information(0, "File type", "I don't know this kind of file, what shall I do ?", "Create a new Filetype", "Read the content of the file", "Cancel", 1, 2);
				if (res == 1) // see the content
				{
					QString commande = hexEditor + " ";
					commande += (char)34;
					commande += name;
					commande += (char)34;
#if !defined(WIN32)
					if (!fork())
					{
#endif
						//char *comm = "c:\\program files\\winzip\\lha.exe";
					
						//int result = system("c:\\progra~1\\winzip\\winzip32.exe");
						//errno;
						system(commande.data());
#if !defined(WIN32)
						exit(127);
					}
#endif
				}
				else if (res == 0) // create a new filetype
				{	  
					CNewFileTypeDialog cf(configFileType, configOptions, name.data(), this);
					cf.setModule(inModule);
					cf.exec();
				}
			}  

		}		
			
			
	}

	/*if (currentDir != QString(""))
	{
		debug("New directory to scan : %s", currentDir.data());
  		setCursor(QCursor(waitCursor));

  		list = inModule -> scan(currentDir);

  		edit -> setText(currentDir);

		if (outModule)
			outModule -> display( list );
		else
		    outGLModule -> display( list );

  		setCursor(QCursor(arrowCursor));
	}*/
}

void ZFenetre::executeCommand(QString command, QString arg)
{
    int res = isInternalCommand(command);
    debug("res = %d", res);

    //if (res == -1) // it's an external command
//	system(command.data());
//    else
//    {
	if (res == 0) // All
	    selectAll();
	else if (res == 1) // AnsiRead
	{
	    internalCommands *p = new internalCommands;
	    p -> textViewer(arg);
	    p -> show();
	}
	else if (res == 2)
	    none();
	else if (res == 3)
	    toggle();
//    }
}


int ZFenetre::isInternalCommand(QString command)
{
    for(int i = 0 ;; i++)
    {
		if (commands[i][0] == 0) break;

		if (command.find(commands[i][0]) == 0) // we got it !!!
			return i;
    }

    return -1;
}


void ZFenetre::highlightEvent(QString name)
{
  	if (name != "..")
    	label -> setText(name);
  	else
    	label -> setText("Parent directory");
    
}
