/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef __ZFENETRE_H_
#define __ZFENETRE_H_

#include <qwidget.h> // class QWidget
#include <qstring.h> // pour class QString
#include <qstrlist.h>
#include <qdir.h>
#include <qlined.h>
#include <qlabel.h>
#include <qevent.h>
#include <qkeycode.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <ctype.h> // isprint()
#include <qpushbutton.h>
#include "./plugins/zcomponent.h"
#include "./plugins/zglcomponent.h"
#include "list.h"
#include "lineedit.h"
#include "cconfig.h"
#include "configOpt.h"
#include "configToolBar.h"
#include "manager.h"
#include "toolbar.h"
#include "threadScan.h"
//#include "threadDisplay.h"

class ZFenetre : public QWidget
{
  Q_OBJECT
   
  public:
    ZFenetre(CConfig *conf1, CConfigOpt *conf2, CConfigToolBar *conf3, QString path, ZManager *man, QString plug, QWidget *parent = 0, const char *name = 0);
    ~ZFenetre();

    QString getCurrentDir();
	QString getCurrentPlugin() { return outPlugin; }

  protected:

	inComponent *inModule;
    outComponent *outModule;
#if !defined(__CYGWIN32__)
    outGLComponent *outGLModule;
#endif

    void *handle;
    QWidget *listeur;

	threadScan *tScan;
	//threadDisplay *tDisplay;
	bool killed;
	//bool stop;
	//int currentItem;
	//int count;

	QTimer timerScan;
	QTimer timerDisplay;

	enum MODE { NONE, SCAN, DISPLAY };
	MODE mode;

    QLabel *label;
    ZLineEdit *edit;    
    ZToolbar *toolbar;

	QPushButton *boutton;
	
    CConfig *configFileType;
    CConfigOpt  *configOptions;
    CConfigToolBar *configToolBar;

    QMenuBar *menu;
    QPopupMenu *fichiers, *options; // menu Fichiers
    QPopupMenu *tri; // menu Tri
    QPopupMenu *inPopup; // menu Options de scan
    QPopupMenu *outPopup; // menu Affichage
	QPopupMenu *plugins; // choix des plugins

    ZList list;
    ZListDevice listDevices;

    //QString dir;
    // repertoire courant

    //QString textEditor;
    QString hexEditor;
    QString pluginPath;
    QString inPlugin;
    QString outPlugin;
    //QString out_Yean3D;
    QString currentDir;

    void resizeEvent(QResizeEvent *);
    void keyPressEvent(QKeyEvent *);
    void closeEvent(QCloseEvent *);

    // internal commands
    void selectAll();
    void none();
    void toggle();
    void executeCommand(QString, QString);
    int isInternalCommand(QString);

    //int currentPlugin;

    void *handleIn, *handleOut;

    ZManager *manager;

  private:
	void changeInModule(int pos);
    void changeOutModule(int pos);	  
#if !defined (__CYGWIN32__)
    void rescanPlugins();
#endif

  public slots:
    void clickEvent(QString, int qualifier);
    void highlightEvent(QString);
    void dblClickEvent(QString, int qualifier);

    void changeInModule(QString name);
    void changeOutModule(QString name);

    void fileMenuClicked(int);
    void inMenuClicked(int);
    void outMenuClicked(int);
    void miscMenuClicked(int);
    void pluginsMenuClicked(int);
	void toolBarClicked(int);

    void tabPressed();

    void enterPressed();

	void killThread();

}; // class ZFenetre

#endif
