/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qapplication.h>
#include "fenetre.h"
#include "manager.h"

QList<ZFenetre> list;

ZManager::ZManager(CConfig *conf1, CConfigOpt *conf2, CConfigToolBar *conf3)
    : config(conf1), configOpt(conf2), configToolBar(conf3)
{
    // scan the option file to open all the needed windows
    QString got;
    QString query;
    QString path;
	QString plugin;

    int x, y, w, h;
    int found1, found2;

    ZFenetre *fen;

    nbWindows = 0;

    for( int i = 0 ;; i++ )
    {
		query.sprintf("plugin%d", i);
		got = configOpt -> get(query, "no more plugins");

		if (got == "no more plugins")
			break;

		outPluginList.append(got);
    }

    for( i = 0 ;; i++ )
    {
		query.sprintf("window%d", i);
		got = configOpt -> get(query, "no more windows");

		if (got == "no more windows")
			break;

		found1 = got.find(',');
		x = got.left(found1).toInt();

		found2 = got.find(',', found1 + 1);
		y = got.mid(found1 + 1, found2 - found1 - 1).toInt();
		found1 = found2;

		found2 = got.find(',', found1 + 1);
		w = got.mid(found1 + 1, found2 - found1 - 1).toInt();
		found1 = found2;

		found2 = got.find(' ', found1 + 1);
		h = got.mid(found1 + 1, found2 - found1 - 1).toInt();
		found1 = found2;

		found2 = got.find(' ', found1 + 1);
		if (found2 == -1)
			path = got.right(got.length() - found1 - 1);
		else
			path = got.mid(found1 + 1, found2 - found1 - 1);
		found1 = found2;

		if (found1 == -1) // no plugin
			plugin = outPluginList.at(0);
		else
			plugin = got.right(got.length() - found1 - 1);

		fen = new ZFenetre(conf1, conf2, conf3, path, this, plugin);
		fen -> setGeometry(x, y, w, h);
		fen -> show();

		list.append(fen);

		nbWindows ++;
    }

    if (nbWindows == 0)
    {
#ifdef WIN32		
		fen = new ZFenetre(conf1, conf2, conf3, "c:\\", this, outPluginList.at(0));
#else
		fen = new ZFenetre(conf1, conf2, conf3, "/", this, outPluginList.at(0));
#endif
		fen -> show();

		list.append(fen);
		nbWindows ++;
    }
}	

void ZManager::rescanPlugins()
{
    QString pluginPath = configOpt -> get("pluginpath", "");
    inPluginList.clear();
	outPluginList.clear();

    QDir d(pluginPath);
    QFileInfo *name;
    void *handle;
    outComponent *outTest;
	inComponent *inTest;

    const QFileInfoList *List = d.entryInfoList();
    QFileInfoListIterator it(*List);

    while( name = it.current() )
    {
		if (name -> isSymLink() == FALSE)
		{
			handle = ZModule::loadModule(name -> absFilePath());
			if (handle > 0)
			{
				outTest = ZModule::getOutModule(handle);
				if (outTest > 0)
					outPluginList.append(name -> baseName());
				else
				{
					outGLComponent *t =	ZModule::getOutGLModule(handle);
					if (t > 0)
						outPluginList.append(name -> baseName());
				}
				inTest = ZModule::getInModule(handle);
				if (inTest > 0)
					inPluginList.append(name -> baseName());
			}
		}
		++it;
    }

    //return pluginList;
}

void ZManager::windowClosed(void *f)
{
	if (nbWindows == 1)
		quit();
    
	ZFenetre *r = (ZFenetre *)f;

    int found = list.find(r);
    if (found >= 0)
		list.remove(found);

    nbWindows --;

    //if (nbWindows == 0)
	///*qApp ->*/ quit();
}

void ZManager::addWindow()
{
    nbWindows ++;

	QString query, got, path;
	int found1, found2;

	query.sprintf("window%d", nbWindows);
	got = configOpt -> get(query, "no more windows");

	if (got != "no more windows")
	{
		found1 = got.find(',');
		
		found2 = got.find(',', found1 + 1);
		found1 = found2;

		found2 = got.find(',', found1 + 1);
		found1 = found2;

		found2 = got.find(' ', found1 + 1);
		found1 = found2;

		path = got.right(got.length() - found2 - 1);
	}
	else
		path = 
#ifdef WIN32
										"c:\\";
#else
										"/";
#endif

    ZFenetre *fen = new ZFenetre(config, configOpt, configToolBar, path.data(), this, outPluginList.at(0));
    fen -> show();

    list.append(fen);
}

void ZManager::quit()
{
    // first, we remove all windows and plugins parameters
    int i = 0;
    QString query, toAdd;

    for(;;)
    {
		query.sprintf("window%d", i++);
		if (configOpt -> removeEntry(query) == FALSE)
			break;
    }

    i = 0;

    for(;;)
    {
		query.sprintf("plugin%d", i++);
		if (configOpt -> removeEntry(query) == FALSE)
			break;
    }

    // then we add all the current windows
    int count = list.count();
    ZFenetre *f;
	QString plug, plug1;
	int found;

    for(i = 0 ; i < count ; i++)
    {
		f = list.at(i);
		query.sprintf("window%d", i);
		plug = f -> getCurrentPlugin();
//#ifdef WIN32
//		found = plug.findRev('\\');
//#else
		found = plug.findRev('/');
//#endif
		plug1 = plug.right(plug.length() - found - 1);

		toAdd.sprintf("%d,%d,%d,%d %s %s", f -> x(), f -> y(), f -> width(), f -> height(), f -> getCurrentDir().data(), plug1.data());

		configOpt -> addEntry(query, toAdd);
    }

    count = outPluginList.count();

    for(i = 0 ; i < count ; i++)
    {
		query.sprintf("plugin%d", i);
		configOpt -> addEntry(query, outPluginList.at(i));
    }
    

    configOpt -> save();
    qApp -> quit();
}


ZManager::~ZManager()
{}
