// classe ZListBox
// (c)1998 GOUIRAN Christophe

//////// Historique //////
//
// 02.11.98 : c'est fait, les plugins fonctionnent !!

#include <qdir.h>
#include <qcursor.h>
#include <qfontmet.h>
#include <qpainter.h>
#include <qkeycode.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qdatetm.h> // class QDateTime
#include <qapp.h> // class QApplication
#include "ZListBox.h"
#include "colordialog.h"

outModule::outModule()
  : outComponent() {}

QWidget *outModule::getNewWidget(CConfig *config, QWidget *parent, const char *name)
{
  component = new ZListBox(config, parent, name);
  return component;
}


void outModule::display(ZList &list)
{
  component -> display(list);
}


void outModule::displayDevices(ZListDevice &list)
{
	component -> displayDevices(list);
}


QStrList outModule::getMenu()
{
	return component -> getMenu();
}


void outModule::menuClicked(int id)
{
	component -> menuClicked(id);
}

extern "C" outComponent
#ifdef WIN32
__declspec(dllexport)
#endif
*getNewOutModule()
{
  return new outModule;
}

ZListBox::ZListBox(CConfig *config, QWidget *parent, const char *name)
	: outComponent(parent, name)
{
	list = new widget(config, this, "liste");
	header = new QHeader(this, "en tete");
	header -> setMovingEnabled(FALSE);
	header -> setClickEnabled(FALSE);

	header -> addLabel("Name");
	header -> addLabel("Size");
	header -> addLabel("FileType");
	header -> addLabel("Hour");
	header -> addLabel("Date");
	header -> addLabel("Permissions");
	header -> addLabel("Owner");
	header -> addLabel("Group");

	scanning = FALSE;

	list -> setMouseTracking(TRUE);

	connect(list, SIGNAL(clicked(QString, int)), SIGNAL(clicked(QString, int)));
	connect(list, SIGNAL(dblClicked(QString, int)), SIGNAL(dblClicked(QString, int)));
	connect(list, SIGNAL(highlighted(QString)), SIGNAL(highlighted(QString)));

	connect(header, SIGNAL(sizeChange(int, int, int)), list, SLOT(sizeChange(int, int, int)));
	// quand on change la taille d'un header, la liste doit changer de largeur aussi

	connect(list, SIGNAL(changeSizeHeader(int, int)), this, SLOT(sizeChange(int, int)));
	// signal inverse (comme ca, pour le cas de la mise a jour automatique, les headers changent de taille
    // apres chaque scan

	connect(list, SIGNAL(offsetChanged(int)), header, SLOT(setOffset(int)));
	// quand on manipule la scrollBar horizontale

	for(int i = 0 ; i < COLS ; list -> sizeChange(i, 0, header -> cellSize(i)), visible[i++] = TRUE);

	autoUpdate = TRUE;
	list -> setFocus();

}


void ZListBox::display(ZList &ll)
{
	header -> setLabel(0, "Name");
	header -> setLabel(1, "Size");

	if (visible[2] == TRUE)
	    header -> setLabel(2, "FileType");
	else
	    header -> setCellSize(2, 0);
	    
	//header -> setCellSize(3, 0);
	//header -> setCellSize(4, 0);
	//header -> setCellSize(5, 0);
	//header -> setCellSize(6, 0);
	
	scanning = TRUE;
	header -> setResizeEnabled(FALSE);
	list -> display(ll, autoUpdate, visible[2]);
	//if (autoUpdate) {header -> setCellSize(0, max);header->repaint();list -> sizeChange(0,0,max);debug("%d", max);}
	scanning = FALSE;
	header -> setResizeEnabled(TRUE);
	header -> repaint();
}


void ZListBox::displayDevices(ZListDevice &ll)
{
	header -> setLabel(0, "Path");
	header -> setLabel(1, "Filesystem");
	header -> setCellSize(2, 0);
	header -> setCellSize(3, 0);
	header -> setCellSize(4, 0);
	header -> setCellSize(5, 0);
	header -> setCellSize(6, 0);

	for(int i = 0 ; i < COLS ; list -> sizeChange(i, 0, header -> cellSize(i)), visible[i++] = TRUE);

	list -> displayDevices(ll, autoUpdate);
}


void ZListBox::resizeEvent(QResizeEvent *e)
{
	header -> setGeometry(0, 0, width(), 20);
	list -> setGeometry(0, 21, width(), height() - 21);
}


QStrList ZListBox::getMenu()
{
	QStrList ll;
	if (visible[0]) // l'icone est visible
		ll.append("Name_1");
	else
		ll.append("Name_0");
	
	if (visible[1]) // la taille est visible
		ll.append("Size_1");
	else
		ll.append("Size_0");
		
	if (visible[2]) // le type de fichiers est visible
		ll.append("FileType_1");
	else
		ll.append("FileType_0");

	if (visible[3]) // l'heure est visible
		ll.append("Hour_1");
	else
		ll.append("Hour_0");

	if (visible[4]) // la date
		ll.append("Date_1");
	else
		ll.append("Date_0");
		
	if (visible[5]) // les permissions
		ll.append("Permissions_1");
	else
		ll.append("Permissions_0");

	if (visible[6]) // le proprio
		ll.append("Owner_1");
	else
		ll.append("Owner_0");
		
	if (visible[7]) // le groupe
		ll.append("Group_1");
	else
		ll.append("Group_0");

	ll.append("---"); // separateur

	if (autoUpdate) // s'il y a la mise a jour automatique
		ll.append("Automatically update cells_1");
	else
		ll.append("Automatically update cells_0");
	
	ll.append("---"); // separateur
	ll.append("Define colors");
	
	return ll;
}


void ZListBox::menuClicked(int id)
{
	switch (id)
	{
		case 0:case 1:case 2:case 3:case 4:case 5:case 6:case 7:

			if (visible[id]) // la colonne etait visible
			{
				largeur[id] = header -> cellSize(id);

				header -> setCellSize(id, 0);
				header -> setResizeEnabled(id, FALSE);
				header -> repaint();
				list -> sizeChange(id, largeur[id], 0);
				visible[id] = FALSE;
			}
			else // la colonne etait invisible
			{
				header -> setCellSize(id, largeur[id]);
				header -> setResizeEnabled(id, TRUE);
				header -> repaint();
				list -> sizeChange(id, 0, largeur[id]);
				visible[id] = TRUE;
			}	

		break;

		case 9: // mise a jour automatique

			if (autoUpdate) autoUpdate = FALSE;
			else autoUpdate = TRUE;

		break;

		case 11: // choix des couleurs

			ZColorDialog *zd = new ZColorDialog(this);

			zd -> addEntry("Couleur de fond", list -> getColor(0));
			zd -> addEntry("Couleur de texte des fichiers", list -> getColor(1));
			zd -> addEntry("Couleur de texte des fichiers selectionnes", list -> getColor(2));
			zd -> addEntry("Couleur de fond des fichiers", list -> getColor(3));
			zd -> addEntry("Couleur de fond des fichiers selectionnes", list -> getColor(4));

			zd -> addEntry("Couleur de texte des repertoires", list -> getColor(5));
			zd -> addEntry("Couleur de texte des repertoires selectionnes", list -> getColor(6));
			zd -> addEntry("Couleur de fond des repertoires", list -> getColor(7));
			zd -> addEntry("Couleur de fond des repertoires selectionnes", list -> getColor(8));

			connect(zd, SIGNAL(colorChanged(int, QColor)), this, SLOT(colorChanged(int, QColor)));
			connect(zd, SIGNAL(sliderReleased()), this, SLOT(sliderReleased()));

			zd -> exec();
			
		break;

		//default:break;
	}
}


void ZListBox::colorChanged(int id, QColor color)
{
	list -> changeColor(id, color);
}


void ZListBox::sliderReleased()
{
	list -> sliderReleased();
}


void ZListBox::sizeChange(int id, int newSize)
{
	header -> setCellSize(id, newSize);
	//if (scanning == FALSE) header -> repaint();
}


#define MAX(a,b) ( (a>b) ? (a) : (b) )
#define MIN(a,b) ( (a<b) ? (a) : (b) )


// la fonction suivante est une surcharge de la fonction originale
// pour pouvoir classer la liste dans le bon ordre
// (sinon elle etait classee dans le sens inverse :( )
#if !defined(__CYGWIN32__) && !defined(WIN32)
int QGList::compareItems(GCI i1, GCI i2)
{
	
	ZFileInfo *fi1 = (ZFileInfo *)i1;
	ZFileInfo *fi2 = (ZFileInfo *)i2;

	return strcmp(fi1 -> fileName().data(), fi2 -> fileName().data());
	
} // compareItems()
#endif

int widget::cellWidth(int n)
{
  return taille[n];
}


int widget::cellHeight(int n)
{
  return fm -> height() + 2;
}


int widget::totalWidth()
{
	int largeur = 0;
	for(int i = 0 ; i < COLS ; largeur += taille[i++]);
	return largeur;
}


int widget::totalHeight()
{
    return (fm -> height() + 2) * nombre;
}


widget::widget(CConfig *config, QWidget *parent, const char *name)
  : QTableView(parent,name), localConfig(config)
{
  	curRow = curCol = numSelected = 0;			// set currently selected cell
  	setFocusPolicy( StrongFocus );	// we accept keyboard focus
    	
    	
  	setTableFlags( Tbl_autoScrollBars |
		 		   Tbl_clipCellPainting |	// avoid drawing outside cell
		 		   Tbl_smoothScrolling);	// easier to see the scrolling
      	
	
  	NombreCol = COLS;

	for(int i = 0 ; i < COLS ; taille[i] = 10, i++);

	font = QFont::defaultFont();
	fm = new QFontMetrics(font);

  	sb = horizontalScrollBar();
  	connect(sb, SIGNAL(valueChanged(int)), SIGNAL(offsetChanged(int)));
  	connect(sb, SIGNAL(sliderMoved(int)), SIGNAL(offsetChanged(int)));

}


void widget::sizeChange(int section, int oldSize, int newSize)
{
	taille[section] = newSize;
	
	updateTableSize();
	repaint();
	
}


void widget::changeColor(int id, QColor coulor)
{
	color[id].setRgb(coulor.red(), coulor.green(), coulor.blue());
}


void widget::sliderReleased()
{
	setBackgroundColor(color[0]);
}


void widget::display( ZList &list, bool autom, bool showft)
{
	setAutoUpdate(FALSE);
	mode = FILES;
	fileTypes.clear();
	//contents.clear();

	//setAutoUpdate(TRUE);
	
	int max[COLS] = {0, 0, 0, 0, 0, 0, 0, 0};
	int i = 0;
  	curRow = curCol = 0;
	setTopCell(0);
	numSelected = 1;

  	contents = list;
  	nombre = contents.count();

	//if (nombre)
	//	contents.at(0) -> setSelected(TRUE);

  	setNumRows(nombre);
	setCellHeight(fm -> height() + 2);

	//setAutoUpdate(TRUE);

	QString temp;

	// recherche des types de fichier (oula que ca va ramer !!!!)
	for (i = 0 ; i < list.count() ; i++)
    {
		//contents.append(list.at(i));
		//nombre = i;
		//setNumRows(i);

	    ZFileInfo *fi = list.at(i);
	    QString name = fi -> absFilePath();
//debug("Valeur de localConfig : %d", localConfig);
	    if (fi -> isDir() == FALSE) 
	    {
	
			fileinfo *res;
			if (showft)
				res = localConfig -> get(name.data());
			else
				res = 0;

			if (res > 0) // on a trouve un type de fichier gere !!!
				fileTypes.append(QString(res -> title));
			else
				fileTypes.append("");
	    }
	    else
			fileTypes.append("");
		
		//for(int k = 0 ; k < COLS ; k++)
		//	updateCell(i, k);
	}

	if (autom) // mise a jour automatique
	{
		for(i = 0 ; i < nombre ; i++)
		{
  			ZFileInfo *fi = list.at(i);

			// le nom
			temp = fi -> fileName();
			if (fi -> isSymLink()) temp += (" -> " + fi -> readLink());    	

			if (max[0] < fm -> width(temp.data())) max[0] = fm -> width(temp.data());

			// la taille
			if (fi -> isDir())
				temp = QString("Dir");
			else
				temp.sprintf("%d", fi -> size());
		
			if (max[1] < fm -> width(temp.data())) max[1] = fm -> width(temp.data());

			// le type de fichier
			if (max[2] < fm -> width(fileTypes.at(i)) && showft) max[2] = fm -> width(fileTypes.at(i));

			// l'heure
			//max[3] = fm -> width("00:00:00");

			// la date
			//max[4] = fm -> width("21/12/1975");

			// les permissions
			//max[5] = fm -> width("rwx rwx rwx");
			
			// le proprietaire
			temp = fi -> owner();
			if (max[6] < fm -> width(temp.data())) max[6] = fm -> width(temp.data());

			// le groupe
			temp = fi -> group();
			if (max[7] < fm -> width(temp.data())) max[7] = fm -> width(temp.data());
			
			//for (int j = 0 ; j < COLS ; j++)
			//{
			//	if (taille[j] != max[j] + 4)
			//	{
			//		if (j == 2 && !showft)
			//			taille[j] = 0;
			//		else
			//		{				
			//			taille[j] = max[j] + 4;	
						//emit changeSizeHeader(i, taille[i]);
			//		}
					// on dit de changer la largeur de l'header No 0
			//	}
			//	updateCell(i, j);
			//}

			//for(int k = 0 ; k < COLS ; k++)
				
		}

		// l'heure
		max[3] = fm -> width("00:00:00");

		// la date
		max[4] = fm -> width("21/12/1975");

		// les permissions
		max[5] = fm -> width("rwx rwx rwx");

		for (int j = 0 ; j < COLS ; j++)
		{
			if (taille[j] != max[j] + 4)
			{
				if (j == 2 && !showft)
				    taille[j] = 0;
				else
				{				
				    taille[j] = max[j] + 4;	
				    //emit changeSizeHeader(i, taille[i]);
				}
				// on dit de changer la largeur de l'header No 0
			}
		}
		//repaint();
		
	}

	for (i = 0 ; i < COLS ; i++)
	{
		emit changeSizeHeader(i, taille[i]);
	}

	

	setAutoUpdate(TRUE);
  	//repaint();
	if (nombre)
		contents.at(0) -> setSelected(TRUE);

	//nombre++;
	//setNumRows(nombre);	
	repaint();
	
}  


void widget::displayDevices(ZListDevice &ll, bool autom)
{
	setAutoUpdate(FALSE);

	
	mode = DEVICES;

	int max[COLS] = {0, 0, 0, 0, 0, 0, 0, 0};
	int i = 0;
  	curRow = curCol = 0;

	contentsDevices = ll;
  	nombre = contentsDevices.count();

	contentsDevices.at(0) -> setSelected(TRUE);

	debug("%d", nombre);

  	setNumRows(nombre);
	setCellHeight(fm -> height() + 2);

	QString temp;

	if (autom)
	{
		for(i = 0 ; i < ll.count() ; i++)
		{
			debug("%s", ll.at(i) -> rootPath().data());
			debug("%s", ll.at(i) -> fileSystem().data());

  			ZFileInfoDevice *fi = contentsDevices.at(i);

			// le path
			temp = fi -> rootPath();

			if (max[0] < fm -> width(temp.data())) max[0] = fm -> width(temp.data());
		
			// le filesystem
			temp = fi -> fileSystem();

			if (max[1] < fm -> width(temp.data())) max[1] = fm -> width(temp.data());



		}

		for (i = 0 ; i < 2; i++)
		{
			if (taille[i] != max[i] + 4)
			{
				taille[i] = max[i] + 4;	
				emit changeSizeHeader(i, taille[i]);
				// on dit de changer la largeur de l'header No 0
			}
		}	

	}

	setAutoUpdate(TRUE);
  	repaint();

}


void widget::resizeEvent(QResizeEvent *r)
{
   	setNumCols(COLS);
   	//setNumRows(nombre);
  
   	QTableView::resizeEvent(r);
}


void widget::paintCell( QPainter* p, int row, int col )
{
	if ((!contents.count() && mode == FILES) || (!contentsDevices.count() && mode == DEVICES)) return;
	//if (contents.count() <= row - 1) return;

	//debug("%d %d", contents.count(), row);

	//debug("paintCell");

	ZFileInfo *fi;
	ZFileInfoDevice *fid;

	QDateTime dt;
	QTime tm;
	QDate dat;

	QColorGroup g = colorGroup();
	
	p -> setPen(color[1]);
 
  	int w,h;
	
  	w = cellWidth( col );			// width of cell in pixels
  	h = cellHeight( row );			// height of cell in pixels

	if (mode == FILES)
	{
  		fi = contents.at(row);

		if (fi -> isSelected())
			p -> fillRect(0, 0, w, h, QApplication::winStyleHighlightColor());	
	}
	else if (mode == DEVICES)
	{
  		fid = contentsDevices.at(row);

		if (fid -> isSelected())
			p -> fillRect(0, 0, w, h, QApplication::winStyleHighlightColor());	

	}

 	QString res;

	if (mode == FILES)
	{	
		dt = fi -> lastModified();
		tm = dt.time();
		dat = dt.date();
	}

	switch (col)
	{
		case 0: // le nom / rootPath

			if (mode == FILES)
			{
				res = fi -> fileName();
				if (fi -> isSymLink()) res += (" -> " + fi -> readLink());    	
			}			
			else if (mode == DEVICES)
				res = fid -> rootPath();	

    		p->drawText( 2, 0, w, h, AlignVCenter, res);

		break;

		case 1: // la taille / le filesystem
			
			if (mode == FILES)
			{
				if (fi -> isDir())
					res = QString("Dir");
				else
					res.sprintf("%d", fi -> size());
			}
			else if (mode == DEVICES)
				res = fid -> fileSystem();

    		p->drawText( 2, 0, w, h, AlignVCenter, res);
		
		break;

		case 2: // le type
			
			if (mode == FILES)
				p -> drawText( 2, 0, w, h, AlignVCenter, fileTypes.at(row));

		break;

		case 3: // l'heure

			if (mode == FILES)
			{
				res.sprintf("%02d:%02d:%02d", tm.hour(), tm.minute(), tm.second());

    			p->drawText( 2, 0, w, h, AlignVCenter, res);
			}

		break;

		case 4: // la date

			if (mode == FILES)
			{
				res.sprintf("%02d/%02d/%02d", dat.day(), dat.month(), dat.year());

    			p->drawText( 2, 0, w, h, AlignVCenter, res);
			}
		break;

		case 5: // les permissions

			if (mode == FILES)
			{
				if (fi -> permission( QFileInfo::ReadUser ))
					res += "r";
				else
					res += "-";

				if (fi -> permission( QFileInfo::WriteUser ))
					res += "w";
				else
					res += "-";
			
				if (fi -> permission( QFileInfo::ExeUser ))
					res += "x";
				else
					res += "-";

				res += " ";

				if (fi -> permission( QFileInfo::ReadGroup ))
					res += "r";
				else
					res += "-";

				if (fi -> permission( QFileInfo::WriteGroup ))
					res += "w";
				else
					res += "-";
			
				if (fi -> permission( QFileInfo::ExeGroup ))
					res += "x";
				else
					res += "-";

				res += " ";

				if (fi -> permission( QFileInfo::ReadOther ))
					res += "r";
				else
					res += "-";

				if (fi -> permission( QFileInfo::WriteOther ))
					res += "w";
				else
					res += "-";
			
				if (fi -> permission( QFileInfo::ExeOther ))
					res += "x";
				else
					res += "-";

    			p->drawText( 2, 0, w, h, AlignVCenter, res);
			}
		break;

		case 6: // proprietaire

			if (mode == FILES)
    		p->drawText( 2, 0, w, h, AlignVCenter, fi -> owner());
			
		break;

		case 7: // groupe

			if (mode == FILES)
    		p->drawText( 2, 0, w, h, AlignVCenter, fi -> group());
			
		break;

		default: break;
	
	}
}


void widget::mouseDoubleClickEvent(QMouseEvent *e)
{
  	QPoint clickedPos = e->pos();		// extract pointer position
  	int oldRow = curRow;			// store previous current cell
  	int oldCol = curCol;
  	curRow = findRow( clickedPos.y() );	// map to row; set current cell
  	curCol = findCol( clickedPos.x() );	// map to col; set current cell
    	
  	if (curRow>nombre || curCol == -1 || curRow < 0)
  	{
    	curRow = oldRow;
    	return;	
  	}
	
  
	if (mode == FILES)
	{
  		if (contents.at(curRow) -> fileName() == "..")
   	 		emit dblClicked(QString(".."), e -> state());
  		else
    		emit dblClicked(contents.at(curRow) -> absFilePath(), e -> state());
	}
	else if (mode == DEVICES)
    	emit dblClicked(contentsDevices.at(curRow) -> rootPath(), e -> state());
		
}


void widget::leaveEvent(QEvent *)
{
  	emit highlighted(QString(""));
}


void widget::deSelect()
{
	unsigned c = contents.count();

	for(unsigned int i = 0 ; i < c ; i++)
		if (contents.at(i) -> isSelected())
		{
			contents.at(i) -> setSelected(FALSE);
			//for(int j = 0 ; j < COLS ; j++)
			//	updateCell(i, j);
		}
}


void widget::mousePressEvent( QMouseEvent* e )
{
	debug("ok");
  
  	if (e -> button() == LeftButton)
  	{
    	int oldRow = curRow;			// store previous current cell
    	int oldCol = curCol;
    	QPoint clickedPos = e->pos();		// extract pointer position
    	curRow = findRow( clickedPos.y() );		// map to row; set current cell
    	curCol = findCol( clickedPos.x() );		// map to col; set current cell
    
    

    	if (curRow>nombre || curCol == -1 || curRow<0)
    	{
      		curRow = oldRow;
      		//emit highlighted(QString(""));
      		deSelect(); // on deselectionne tous les fichiers
      		return;	
    	}
    	
		if (mode == FILES) 
		{
    		if ((e->state() & ControlButton) != 0 ) // on appuie sur control en meme temps ???
    		{    
       			if (contents.at(curRow)->isSelected())
				{
	   				contents.at(curRow)->setSelected(FALSE);
	 				numSelected--;
				}
				else
				{ 
					contents.at(curRow)->setSelected(TRUE);
					numSelected++;
				}

				for(int i = 0 ; i < COLS ; i++)
					updateCell(curRow, i);
    		}

    		if ((e->state() & ShiftButton) != 0 ) // on appuie sur shift en meme temps ???
    		{	   
	 			//deSelect(); // on deselectionne tous les autres fichiers
				debug("Boutton shift");
	 			int min = MIN(lastClickedY, curRow);
	 			int max = MAX(lastClickedY, curRow);
			
				debug("min max = %d %d", min, max);

				numSelected = 0;

				int topcell = topCell();
				int lastrowvisible = lastRowVisible();

				for (int i = 0 ; i < nombre ; i++)
				{
					bool old = contents.at(i) -> isSelected();

					if (i >= min && i <= max)
						contents.at(i)->setSelected(TRUE);
					else			
						contents.at(i)->setSelected(FALSE);

					if (i <= lastrowvisible)
						for(int j = 0 ; j < COLS ; j++)
							updateCell(i, j);
				}

				numSelected = max - min + 1;

    		}

    		if (!e->state()) // on appuie sur aucune touche en meme temps
    		{
	 			//deSelect(); // on deselectionne tous les autres fichiers

	 			lastClickedX = 0; // on garde trace d'ou
	 			lastClickedY = curRow; // on a clique (pour la touche shift plus tard)

				numSelected = 1;

				int topcell = topCell();
				int lastrowvisible = lastRowVisible();

				//debug("topcell : %d", topcell);
				//debug("lastrow : %d", lastrowvisible);

				for (int i = 0 ; i < nombre ; i++)
				{
					bool old = contents.at(i) -> isSelected();

					if (i == curRow)
					{
	 					if (contents.at(curRow)->isSelected())
	   						contents.at(curRow)->setSelected(FALSE);
	 					else 
							contents.at(curRow)->setSelected(TRUE);
					}
					else
						contents.at(i) -> setSelected(FALSE);

					if ((old == TRUE && i <= lastrowvisible) || i == curRow)
						for(int j = 0 ; j < COLS ; j++)
							updateCell(i, j);
				}
	 
    		}

	 		//updateCell( oldRow, 0);		// erase previous marking
	 		//updateCell( curRow, 0);
		}  	
	}

	if (mode == FILES)
	{
  		if (contents.at(curRow) -> fileName() == "..")
    		emit clicked(QString(".."), e -> state());
  		else
    		emit clicked(contents.at(curRow) -> absFilePath(), e -> state());
	}
	else if (mode == DEVICES)
   		emit dblClicked(contentsDevices.at(curRow) -> rootPath(), e -> state());
          
}


void widget::mouseMoveEvent(QMouseEvent *e)
{
  	int row,col;
  	col = findCol( e->x() );
  	row = findRow( e->y() );
		
  	if (col<numCols() && row<numRows() && col!=-1 && row !=-1)
  	{
		if (mode == FILES)
		{
      		if (contents.at(row) -> fileName() == "..")
				emit highlighted(QString(".."));
      		else
				emit highlighted(contents.at(row) -> filePath());
		}
		else if (mode == DEVICES)
			emit highlighted(contentsDevices.at(row) -> rootPath());
	}
  	else 
    	emit highlighted(QString(""));

}


void widget::keyPressEvent( QKeyEvent* e )
{
   	int oldRow = curRow;			// store previous current cell
   	int oldCol = curCol;
   	int edge = 0;
   	switch( e->key() ) 
	{			// Look at the key code
	
		case Key_Up:
	    	if( curRow > 0 ) 
			{
				if (numSelected == 1) 
				{	
					contents.at(curRow) -> setSelected(FALSE);
					//for(int i = 0 ; i < COLS ; i++) updateCell(curRow, i);
				}

				curRow--;
				if (numSelected > 1) deSelect();
				numSelected = 1;
				contents.at(curRow) -> setSelected(TRUE);
				edge = topCell();
				if ( curRow < edge )
		    		setTopCell( edge - 1 );
	    	}
	    
			break;
	
		case Key_Down:
	    	if( curRow < numRows()-1 ) 
			{
				if (numSelected == 1) 
				{	
					contents.at(curRow) -> setSelected(FALSE);
					//for(int i = 0 ; i < COLS ; i++) updateCell(curRow, i);
				}

				curRow++;
				if (numSelected > 1) deSelect();
				numSelected = 1;
				contents.at(curRow) -> setSelected(TRUE);
				edge = lastRowVisible();
				if ( curRow >= edge )
		    		setTopCell( topCell() + 1 );
	    	}
	    
			break;
	
		case Key_Return:case Key_Enter:
	debug("enter pressed");
			if (mode == FILES)
	    		emit dblClicked(contents.at(curRow) -> absFilePath(), e -> state());
	    	else if (mode == DEVICES)
			{debug("mode == DEVICES : %d", curRow);debug("%s", contentsDevices.at(curRow) -> rootPath().data());
	    		emit dblClicked(contentsDevices.at(curRow) -> rootPath(), e -> state());
}
			break;break;
	
	
		default:				// If not an interesting key,
	    	e->ignore();			// we don't accept the event
	    
		return;	
    }
    
    if ( (curRow != oldRow) 			// if current cell has moved,
	 || (curCol != oldCol)  ) 
	{
		for(int i = 0 ; i < COLS ; i++)
		{
			updateCell( curRow, i );		// erase previous marking
			updateCell( oldRow, i );
		}
		// show new current cell
    }
}


void widget::focusInEvent( QFocusEvent* )
{
	for(int i = 0 ; i < NombreCol ; i++) 
		updateCell( curRow, i );		// draw current cell
}   


void widget::focusOutEvent( QFocusEvent* )
{
    for(int i = 0 ; i < NombreCol ; i++) 
		updateCell( curRow, i );		// draw current cell
} 
