/*
************************************************************
 
DirIcn, a View Plugin for ZFM,
http://www.multimania.com/cgouiran/ZFM/zfm.htm
 
This Plugin displays an icon-classic view
author	: Nicolas Roard, 1999
date 	: 17/09/99
version : 0.3
New versions of this plugin will be 
available on http//zfm.xdev.org
 
TODO 	: add config dialogues
	  add multiselection
************************************************************
*/

#include <qdir.h>
#include <qcursor.h>
#include <qfontmet.h>
#include <qpainter.h>
#include <qkeycode.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qdatetm.h>
#include <qapp.h>
#include <iostream.h>
#include "dirIcn.h"
#include <qprinter.h>
#include <qpixmap.h>

#ifdef WIN32
#include <windows.h>
#endif

// Some defines ...

#define TYPE_DIR	1
#define TYPE_LINKDIR	2
#define TYPE_FILE	3
#define TYPE_EXE	4
#define TYPE_LINK	5

#define TYPE_DIRROOT	6
#define TYPE_DIRHOME	7
#define TYPE_DIRPARENT	8
#define TYPE_CORE	9

#define ICON_WIDTH	90
#define ICON_HEIGHT	90

outModule::outModule() : outComponent()
{}

QWidget *outModule::getNewWidget(CConfig *config, QWidget *parent, const char *name)
{
    component = new MonPlug(config, parent, name);
    return component;
}

void outModule::display(ZList &list)
{
    component -> display(list);
}
void outModule::displayDevices(ZListDevice &list)
{
    component -> displayDevices(list);
}
QStrList outModule::getMenu()
{
    return component -> getMenu();
}
void outModule::menuClicked(int id)
{
    component -> menuClicked(id);
}

extern "C" outComponent
#ifdef WIN32
            __declspec(dllexport)
#endif
            *getNewOutModule()
{
    return new outModule;
}

MonPlug::MonPlug (CConfig *config, QWidget * parent, const char *name)
        : outComponent (parent, name)
{
    widget = new DirIcn(config, this, "widget");
    connect(widget, SIGNAL(clicked(QString, int)), SIGNAL(clicked(QString, int)));
    connect(widget, SIGNAL(dblClicked(QString, int)), SIGNAL(dblClicked(QString, int)));
    connect(widget, SIGNAL(highlighted(QString)), SIGNAL(highlighted(QString)));
}

void MonPlug::resizeEvent(QResizeEvent *e)
{
    widget -> setGeometry(0, 0, width(), height());
}
void MonPlug::display(ZList &list)
{
    widget -> display(list);
}
void MonPlug::displayDevices(ZListDevice &list)
{
    widget -> displayDevices(list);
}
QStrList MonPlug::getMenu()
{
    return widget -> getMenu();
}
void MonPlug::menuClicked(int id)
{
    widget -> menuClicked(id);
}

////////////////////////////////////////////
// A little manipulation class, listView  //
////////////////////////////////////////////

listView::listView (ZList &list)
{

    //debug ("On est dans listView \n");

    m_NbFiles = 0;
    m_listType = 0;
    m_fileNames = 0;
    m_Paths = 0;
    m_oldRow = 0;
    m_oldCol = 0;

    setList(list);
    //debug ("On quitte listView \n");
}

listView::~listView ()
{
    //debug ("on est dans le destructeur listView \n");
    if (m_listType)
        delete [] m_listType;
    if (m_fileNames)
        delete [] m_fileNames;
    if (m_Paths)
        delete [] m_Paths;
    if (m_oldRow)
        delete [] m_oldRow;
    if (m_oldCol)
        delete [] m_oldCol;
    //debug ("on quitte le destructeur listView \n");
}

int listView::setList (ZList &list)
{
    m_NbFiles = list.count();

    //debug("On est dans setList , il y a %i fichiers \n", m_NbFiles);

    if (m_NbFiles > 0)
    {

        // We destroy old tabs

        if (m_listType)
            delete [] m_listType;
        if (m_fileNames)
            delete [] m_fileNames;
        if (m_Paths)
            delete [] m_Paths;
        if (m_oldRow)
            delete [] m_oldRow;
        if (m_oldCol)
            delete [] m_oldCol;

        // ...and create some news

        m_listType = new int [m_NbFiles];
        m_fileNames = new QString [m_NbFiles];
        m_Paths = new QString [m_NbFiles];
        m_oldRow = new int[m_NbFiles];
        m_oldCol = new int[m_NbFiles];

        // Now we could fill theses new tabs :
        // filenames, filetypes...

        if (m_NbFiles > 0)
        {

            // we've got the path !

            for (int i = 0; i < m_NbFiles; i++)
            {

                ZFileInfo *f = list.at(i);

                // get the filename

                cout << list.at(i)->fileName() << endl;
                m_fileNames[i] = list.at(i)->fileName();
                m_Paths[i] = list.at(i)->absFilePath();
                m_oldRow[i] = -1;
                m_oldCol[i] = -1;

                // find the filetype [I MUST CHANGE THIS!]

                if (f->isDir())
                {
                    m_listType[i] = TYPE_DIR;
                    if (f->isSymLink())
                        m_listType[i] = TYPE_LINKDIR;
                    if (f->fileName().stripWhiteSpace() == "root")
                        m_listType[i] = TYPE_DIRROOT;
                    if (f->fileName().stripWhiteSpace() == "home")
                        m_listType[i] = TYPE_DIRHOME;
                    if (f->fileName().stripWhiteSpace() == "..")
                        m_listType[i] = TYPE_DIRPARENT;
                }
                else
                {
                    m_listType[i] = TYPE_FILE;
                    if (f->permission(QFileInfo::ExeUser))
                        m_listType[i] = TYPE_EXE;
                    if (f->isSymLink())
                        m_listType[i] = TYPE_LINK;
                    if (f->fileName().stripWhiteSpace() == "core")
                        m_listType[i] = TYPE_CORE;
                }


            }
        }
    }
    //debug("On quitte setList , il y a %i fichiers \n", m_NbFiles);
    return 0;
}

int listView::getType (QString Name)
{
    return - 1;
}

// Accesseurs / Modifieurs

int listView::getOldRow (int Num)
{
    int result = -1;
    if ((Num < m_NbFiles) && (Num >= 0))
        if (m_oldRow)
            result = m_oldRow[Num];
    return result;
}

int listView::getOldCol (int Num)
{
    int result = -1;
    if ((Num < m_NbFiles) && (Num >= 0))
        if (m_oldCol)
            result = m_oldCol[Num];
    return result;
}

int listView::setOldRow (int Num, int row)
{
    if ((Num < m_NbFiles) && (Num >= 0))
        if (m_oldRow)
		{		
            m_oldRow[Num] = row;
			return row;
		}
		

	return 0;
}

int listView::setOldCol(int Num, int col)
{
    if ((Num < m_NbFiles) && (Num >= 0))
        if (m_oldCol)
		{
            m_oldCol[Num] = col;
			return col;
		}
		
	return 0;
}

int listView::getType (int Num)
{
    //debug ("On est dans listview getType \n");
    int result = -1;
    if ((Num < m_NbFiles) && (Num >= 0))
        if (m_listType)
            result = m_listType[Num];
    //debug ("On quitte listview getType \n");
    return result;
}

QString listView::getName (int Num)
{
    //debug ("On est dans listview getName (%i) , m_NbFiles = %i \n", Num, m_NbFiles);
    QString result = "";
    if ((Num < m_NbFiles) && (Num >= 0))
        if (m_fileNames)
            result = m_fileNames[Num];
    //debug ("On quitte listview getName \n");
    return result;
}

QString listView::getPath (int Num)
{
    QString result = "";
    if ((Num < m_NbFiles) && (Num >= 0))
        if (m_Paths)
            result = m_Paths[Num];
    return result;
}

int listView::getNbFiles ()
{
    return m_NbFiles;
}

////////////////////////////////////////////
// My own class/plugin, DirIcn...         //
////////////////////////////////////////////

void DirIcn::display(ZList &ll)
{
    //debug("On est dans DirIcn display \n");

    contents = ll;

    int largeurW = (width() / ICON_WIDTH) ;
    int hauteurW = (contents.count() / largeurW) + 1 ;

    setLargeur(largeurW);
    setHauteur(hauteurW);

    if (IconList)
    {
        ////debug("IconList existe, on fait juste un setList() \n");
        IconList->setList(contents);
    }
    else
    {
        ////debug("IconList n'existe pas, on le cree \n");
        IconList = new listView(contents);
    }

    updateSize();
    update();
    //debug("On quitte DirIcn display \n");
}

// This function is not again well-implemented in ZFM, so...
void DirIcn::displayDevices(ZListDevice &ll)
{
    cout << "on est dans displayDevices" << endl;
}

// This is our menu

QStrList DirIcn::getMenu()
{
    //debug("on est dans getMenu() \n");
    QStrList menu;
    menu.append("Icons Configuration");
    menu.append("---");
    menu.append("About");
    //debug("on quitte getMenu() \n");
    return menu;
}

// Hmm... not again implemented ...

void DirIcn::menuClicked(int id)
{}
void DirIcn::colorChanged(int id, QColor color)
{}
void DirIcn::sliderReleased()
{}

// some others functions

void DirIcn::setWidthIcones (int width)
{
    setCellWidth(width);
}

void DirIcn::setHeightIcones (int height)
{
    setCellHeight(height);
}

void DirIcn::setLargeur (int largeur)
{
    if (largeur > 0)
    {
        curRow = curCol = 0;
        m_Largeur = largeur;
    }
}

void DirIcn::setHauteur (int hauteur)
{
    if (hauteur > 0)
    {
        curRow = curCol = 0;
        m_Hauteur = hauteur;
    }
}

void DirIcn::updateSize ()
{
    if ((m_Largeur > 0) && (m_Hauteur > 0))
    {
        setNumRows(getHauteur());
        setNumCols(getLargeur());
    }
}

int DirIcn::getLargeur ()
{
    return m_Largeur;
    //return numCols();
}
int DirIcn::getHauteur ()
{
    return m_Hauteur;
    //return numRows();
}


DirIcn::DirIcn(CConfig *config, QWidget *parent, const char *name )
        : QTableView(parent, name)
{
    //debug("Constructeur DirIcn \n");
    curRow = curCol = 0;
    setFocusPolicy( StrongFocus );
    setBackgroundMode( PaletteBase );
    setTableFlags( Tbl_autoScrollBars | Tbl_clipCellPainting | Tbl_smoothScrolling);
    setMinimumWidth(ICON_WIDTH);
    setMinimumHeight(ICON_HEIGHT);
    setMouseTracking(TRUE);

    curRow = 0;
    curCol = 0;
    m_Largeur = 0;
    m_Hauteur = 0;

    IconList = 0;

    QString ConfigFile;
    QString IconsDir;
#if defined(WIN32)
	char path[MAX_PATH + 1];
	GetWindowsDirectory(path, MAX_PATH);
	ConfigFile = path;
	IconsDir = path;

	ConfigFile	+= "\\.zfm\\icons.cfg";
    IconsDir += "\\.zfm\\icons\\";
#else	
	ConfigFile = getenv("HOME");
	IconsDir = getenv("HOME");
	ConfigFile	+= "/.zfm/icons.cfg";
    IconsDir += "/.zfm/icons/";
#endif
    
	
	
	debug("%s", ConfigFile.data());
	debug("%s", IconsDir.data());

    configOptions = new CConfigOpt(ConfigFile, QDir::current());

    // Classics types

    PIXfolderClose = new QPixmap (IconsDir + configOptions->get("FolderClose", "folder3-blue.xpm"));
    PIXfolderOpen = new QPixmap (IconsDir + configOptions->get("FolderOpen", "folder4-blue.xpm"));
    PIXfolderLink = new QPixmap (IconsDir + configOptions->get("FolderLink", "folder4-blue-link.xpm"));
    PIXfileText = new QPixmap (IconsDir + configOptions->get("FileText", "generic-text.xpm"));
    PIXfileExec = new QPixmap (IconsDir + configOptions->get("FileExec", "file-executable.xpm"));
    PIXfileLink = new QPixmap (IconsDir + configOptions->get("FileLink", "file-link.xpm"));

    // 'Named' types

    PIXfolderRoot	= new QPixmap (IconsDir + configOptions->get("FolderRoot", ""));
    PIXfolderHome	= new QPixmap (IconsDir + configOptions->get("FolderHome", ""));
    PIXfolderParent	= new QPixmap (IconsDir + configOptions->get("FolderParent", ""));
    PIXfileCore	= new QPixmap (IconsDir + configOptions->get("FileCore", ""));
    PIXfileDefault	= new QPixmap (IconsDir + configOptions->get("FileDefault", ""));

    if (configOptions->get("FocusMouse", "0").stripWhiteSpace() == "1")
        FocusMouse = TRUE;
    else
        FocusMouse = FALSE;

    if (configOptions->get("OneClick", "1").stripWhiteSpace() == "1")
        OneClick = TRUE;
    else
        OneClick = FALSE;

    setCellWidth(ICON_WIDTH);
    setCellHeight(ICON_HEIGHT);

    setAutoUpdate (TRUE);
    //debug("On quitte le Constructeur DirIcn \n");
}

void DirIcn::paintCell( QPainter* p, int row, int col )
{

    //debug ("On est dans paintcell(%i,%i) \n", row, col);

    if ((row < getHauteur()) && (col < getLargeur()))
    {
        int Num = (row * getLargeur()) + col;

        //if ((IconList -> getOldRow(Num) != row) | (IconList -> getOldCol(Num) != col))
        //{
        IconList->setOldRow(Num, row);
        IconList->setOldCol(Num, col);

        int w = cellWidth( col );
        int h = cellHeight( row );
        int posPixX;

        //////debug (" row : %i, col : %i, getLargeur() : %i, Num = %i \n", row, col, getLargeur(), Num);

        //p->eraseRect(0, 0, w, h);

        posPixX = (w - PIXfolderClose->width()) / 2;

        if (IconList->getType(Num) > 0)
        {
            switch (IconList->getType(Num))
            {
                case TYPE_DIR:
                    if ( (row == curRow) && (col == curCol) ) 	// if we are on current cell
                        p->drawPixmap( posPixX, 0, *PIXfolderOpen);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfolderOpen);
                    else
                        p->drawPixmap( posPixX, 0, *PIXfolderClose);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfolderClose);
                    break;
                case TYPE_LINKDIR:
                    p->drawPixmap( posPixX, 0, *PIXfolderLink);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfolderLink);
                    break;
                case TYPE_FILE:
                    p->drawPixmap( posPixX, 0, *PIXfileText);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfileText);
                    break;
                case TYPE_EXE:
                    p->drawPixmap( posPixX, 0, *PIXfileExec);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfileExec);
                    break;
                case TYPE_LINK:
                    p->drawPixmap( posPixX, 0, *PIXfileLink);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfileLink);
                    break;
                case TYPE_DIRROOT:
                    p->drawPixmap( posPixX, 0, *PIXfolderRoot);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfolderRoot);
                    break;
                case TYPE_DIRHOME:
                    p->drawPixmap( posPixX, 0, *PIXfolderHome);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfolderHome);
                    break;
                case TYPE_DIRPARENT:
                    p->drawPixmap( posPixX, 0, *PIXfolderParent);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfolderParent);
                    break;
                case TYPE_CORE:
                    p->drawPixmap( posPixX, 0, *PIXfileCore);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfileCore);
                    break;
                default:
                    p->drawPixmap( posPixX, 0, *PIXfileDefault);
                    //bitBlt(this, (col*ICON_WIDTH)+posPixX,(row*ICON_HEIGHT),PIXfileDefault);
            }

            //  }
            if ((row == curRow) && (col == curCol))
            {
                p->fillRect(15, 58, w - 15, 18, QBrush(QColor(100, 0, 250)));
                p->setPen(QColor(255, 255, 255));
            }

            QString FileText = QString(IconList->getName(Num)).stripWhiteSpace();
            QRect RectText = p->boundingRect (10, 50, w - 10, 30, AlignCenter, FileText);
            if (RectText.width() > (w - 20))
            {
                int CarWidth = 8;
                int MaxCars = (w - 10) / CarWidth;
                if (MaxCars > FileText.length()) MaxCars = FileText.length();
                FileText = FileText.left((MaxCars - 3) / 2) + "..." + FileText.right((MaxCars - 3) / 2);
            }
            p->drawText( 10, 50, w - 10, 30, AlignCenter, FileText );
            p->setPen(QColor(0, 0, 0));

            //////debug ("On quitte paintcell(%i,%i) \n", row, col);
        }
    }
}


void DirIcn::mousePressEvent( QMouseEvent* e )
{
    int oldRow = curRow;             			// store previous current cell
    int oldCol = curCol;
    QPoint clickedPos = e->pos();             		// extract pointer position
    curRow = findRow( clickedPos.y() );             		// map to row; set current cell
    curCol = findCol( clickedPos.x() );             		// map to col; set current cell
    if ( (curRow != oldRow) 			// if current cell has moved,
                || (curCol != oldCol) )
    {
        updateCell( oldRow, oldCol );             		// erase previous marking
        updateCell( curRow, curCol );             		// show new current cell
    }

    int Num = (curRow * getLargeur()) + curCol;

    if (!OneClick)
    {
        emit clicked (IconList->getPath(Num), e->state());
    }
    else
    {
        emit dblClicked (IconList->getPath(Num), e->state());
    }
    emit highlighted (IconList->getPath(Num));
}

void DirIcn::mouseDoubleClickEvent ( QMouseEvent* e )
{
    int oldRow = curRow;             			// store previous current cell
    int oldCol = curCol;
    QPoint clickedPos = e->pos();             		// extract pointer position
    curRow = findRow( clickedPos.y() );             		// map to row; set current cell
    curCol = findCol( clickedPos.x() );             		// map to col; set current cell
    if ( (curRow != oldRow) 			// if current cell has moved,
                || (curCol != oldCol) )
    {
        updateCell( oldRow, oldCol );             		// erase previous marking
        updateCell( curRow, curCol );             		// show new current cell
    }
    int Num = (curRow * getLargeur()) + curCol;
    emit dblClicked (IconList->getPath(Num), e->state());
}

void DirIcn::mouseMoveEvent( QMouseEvent* e )
{

    if (FocusMouse)
    {
        int oldRow = curRow;             			// store previous current cell
        int oldCol = curCol;
        curRow = findRow( e->y() );             		// map to row; set current cell
        curCol = findCol( e->x() );             		// map to col; set current cell
        if ( (curRow != oldRow) 			// if current cell has moved,
                    || (curCol != oldCol) )
        {
            updateCell( oldRow, oldCol );             		// erase previous marking
            updateCell( curRow, curCol );             		// show new current cell
        }
        int Num = (curRow * getLargeur()) + curCol;
        emit highlighted (IconList->getPath(Num));
    }
    else
    {
        int Num = (findRow(e->y()) * getLargeur()) + findCol(e->x());
        emit highlighted (IconList->getPath(Num));
    }
}

void DirIcn::keyPressEvent( QKeyEvent* e )
{
    int oldRow = curRow;             			// store previous current cell
    int oldCol = curCol;
    int edge = 0;
    switch ( e->key() )
    {			// Look at the key code
        case Key_Left:             				// If 'left arrow'-key,
            if ( curCol > 0 )
            {			// and cr't not in leftmost col
                curCol--;                  			// set cr't to next left column
                edge = leftCell();             		// find left edge
                if ( curCol < edge )		// if we have moved off  edge,
                    setLeftCell( edge - 1 );             	// scroll view to rectify
            }
            break;
        case Key_Right:             				// Correspondingly...
            if ( curCol < numCols() - 1 )
            {
                curCol++;
                edge = lastColVisible();
                if ( curCol >= edge )
                    setLeftCell( leftCell() + 1 );
            }
            break;
        case Key_Up:
            if ( curRow > 0 )
            {
                curRow--;
                edge = topCell();
                if ( curRow < edge )
                    setTopCell( edge - 1 );
            }
            break;
        case Key_Down:
            if ( curRow < numRows() - 1 )
            {
                curRow++;
                edge = lastRowVisible();
                if ( curRow >= edge )
                    setTopCell( topCell() + 1 );
            }
            break;
        default:             				// If not an interesting key,
            e->ignore();             			// we don't accept the event
            return;
    }

    if ( (curRow != oldRow) 			// if current cell has moved,
                || (curCol != oldCol) )
    {
        updateCell( oldRow, oldCol );             		// erase previous marking
        updateCell( curRow, curCol );             		// show new current cell
    }
}

void DirIcn::focusInEvent( QFocusEvent* )
{
    updateCell( curRow, curCol );             		// draw current cell
}

void DirIcn::focusOutEvent( QFocusEvent* )
{
    updateCell( curRow, curCol );             		// draw current cell
}

void DirIcn::resizeEvent(QResizeEvent *r)
{
    //debug ("on est dans resizeEvent \n");

    if (IconList)
    {
        if (IconList->getNbFiles())
        {
            //////debug (" il y a %i fichiers dans IconList \n", IconList->getNbFiles());
            //int largeurW = (width() / ICON_WIDTH) ;
            //int hauteurW = ( IconList->getNbFiles() / largeurW ) ;

            setNumRows(getHauteur() + 1);
            setNumCols(getLargeur() + 1);
            ZList tampon = contents;
            display(tampon);
        }
    }
    //debug ("on quitte resizeEvent \n");
}
