#ifndef __FILE_H__
#define __FILE_H__

#include <qgl.h>
#include <qapp.h>
#include <math.h>
#include <stdlib.h>
#include <qlist.h>
#include <qlabel.h>
#include <qdir.h>
#include <qscrbar.h>
#if !defined(WIN32)
#include <unistd.h>
#endif
#include <qtimer.h>
#include "outGLModule.h"

// Une classe de vectuer 4D : (mais elle n'est pas utilisee, me semble-t-il)
struct Vector 
{
   // Constructeur par defaut :
   Vector( GLfloat X = 0, GLfloat Y = 0, GLfloat Z = 0, GLfloat W = 0 );
   // Constructeur par recopie :
   Vector( const Vector& );
   
   // Accesseurs :
   GLfloat x() const;
   GLfloat y() const;
   GLfloat z() const;
   GLfloat w() const;
   GLfloat* coordinates();
   void setX( GLfloat );
   void setY( GLfloat );
   void setZ( GLfloat );
   void setW( GLfloat );
   void setCoordinates( GLfloat* );
    
   // Casting operators
   operator GLfloat*();
   // Other operators
   Vector& operator =( const Vector& );
   Vector& operator +=( const Vector& );
   Vector& operator *=( GLfloat );
   
 protected:
   GLfloat ord[4];
};

// Stuct Vector methods...
inline Vector::Vector( GLfloat X, GLfloat Y, GLfloat Z, GLfloat W ) {
   ord[0] = X;
   ord[1] = Y;
   ord[2] = Z;
   ord[3] = W;
};

inline GLfloat Vector::x() const {
   return ord[0];
}

inline GLfloat Vector::y() const {
   return ord[1];
}

inline GLfloat Vector::z() const {
   return ord[2];
}

inline GLfloat Vector::w() const {
   return ord[3];
}

inline GLfloat* Vector::coordinates() {
   return ord;
}

inline void Vector::setX( GLfloat f ) {
   ord[0] = f;
}

inline void Vector::setY( GLfloat f ) {
   ord[1] = f;
}

inline void Vector::setZ( GLfloat f ) {
   ord[2] = f;
}

inline void Vector::setW( GLfloat f ) {
   ord[3] = f;
}

inline void Vector::setCoordinates( GLfloat* f ) {
   ord[0] = f[0];
   ord[1] = f[1];
   ord[2] = f[2];
   ord[3] = f[3];
}

inline Vector::Vector( const Vector& vec ) {
   ord[0] = vec.x();
   ord[1] = vec.y();
   ord[2] = vec.z();
   ord[3] = vec.w();
}
   
inline Vector::operator GLfloat*() {
   return coordinates();
}

inline Vector& Vector::operator =( const Vector& vec ) {
   ord[0] = vec.x();
   ord[1] = vec.y();
   ord[2] = vec.z();
   ord[3] = vec.w();
   return *this;
}

inline Vector& Vector::operator +=( const Vector& vec ) {
   ord[0] += vec.x();
   ord[1] += vec.y();
   ord[2] += vec.z();
   ord[3] += vec.w();
   return *this;
}

inline Vector& Vector::operator *=( GLfloat scale ) {
   ord[0] *= scale;
   ord[1] *= scale;
   ord[2] *= scale;
   ord[3] *= scale;
   return *this;
}




struct BoxInfo {
   BoxInfo(GLfloat X = 0, GLfloat Y = 0, GLfloat Z = 0, int col = 0, QString n = "" )
     { x = X; y = Y; z = Z; colour = col; name = n; }
   GLfloat x,y,z;
   int colour; // Index into the above colour array, using above colour enums
   QString name;
};

class FileWidget : public outGLComponent
{
   Q_OBJECT
 public:
   FileWidget( CConfig *f, QWidget *parent=0, const char *name=0 );

   void display(ZList &list);
   void displayDevices(ZListDevice &list);
   QStrList getMenu();

   // List of boxes:
   QList<BoxInfo> boxInfo;
   uint selbox, firstbox;

   // State controls:
   void setMovement( bool );  // set to true if you want things to move, else they wont
   
 public slots:
     void update();
     // appellee a chaque fois qu'il faut recalculer la vue
     // associee avec le timer interne

     void setTimer(bool n);
     // TRUE -> reamorce le timer
     // FALSE -> arrete le timer

	void menuClicked(int);

 signals:
     void highlighted(QString);
     void clicked(QString, int);
     void dblClicked(QString, int);
   
 protected:
   void resizeGL( int, int );
   void paintGL();
   //void timerEvent( QTimerEvent * );
   void mousePressEvent( QMouseEvent * );
   void mouseReleaseEvent( QMouseEvent * );
   void mouseMoveEvent( QMouseEvent * );

   void focusInEvent(QFocusEvent *);
   void focusOutEvent(QFocusEvent *);

   void updateSelection();
   Vector screenToPlane(const Vector&);
   
   // For moving with the mouse
   bool dragging;
   QPoint dragpos;

   QTimer *timer;
   // necessaire pour savoir quand on doir updater la vue (et pour ne pas consommer du temps CPU pour rien)

   bool timerActive;
   // pour savoir s'il faut recalculer la vue

   // ID of the box display list
   GLint file1, sel1;
   
   // My rotation, position, and velocity
   Vector rot, pos, vel;
   // Position of the mouse
   QPoint mouse;

   // Selection...
   GLfloat selx, sely;
   
   // State info
   bool first_time;
   GLfloat state, angle;
   uint movesel; // saved selection for moving stuff
   
   // Procedures d'affichage de certains elements graphiques :
   void DrawLight(); // Active la lumiere

   QStrList local;
   QDir dir;

  private:
    int time;
};

#endif // __FILE_H__
